program Plat;

uses
	Dos, Objects, Drivers, Memory, Views, Menus, Dialogs, StdDlg, MsgBox, App,
	DemoCmds, Gadgets, Cale, Calc, FileServ, pltCMD, PltDR, Recode;

type

	PPlat = ^TPlat;
	TPlat = object (TApplication)
		Clock		: PClockView;
		Heap		: PHeapView;
		constructor	Init;
		procedure	FileOpen (S : PathStr);
		procedure	Work (FileName : PathStr);
		procedure	GetEvent (Var Event : TEvent); virtual;
		procedure	HandleEvent (Var Event : TEvent); virtual;
		procedure	Idle; virtual;
		procedure	InitMenuBar; virtual;
		procedure	InitStatusLine; virtual;
		procedure	OutOfMemory; virtual;
	end;

constructor	TPlat.Init;
	var
		R		: TRect;
		I		: Integer;
		FileName: PathStr;
	begin
		TApplication.Init;
		GetExtent(R);
		R.A.X := R.B.X - 9; R.B.Y := R.A.Y + 1;
		Clock := New (PClockView, Init (R));
		Insert (Clock);

		GetExtent (R);
		Dec (R.B.X);
		R.A.X := R.B.X - 9; R.A.Y := R.B.Y - 1;
		Heap := New (PHeapView, Init (R));
		Insert (Heap);
	end;


procedure TPlat.GetEvent (Var Event : TEvent);
begin
	TApplication.GetEvent (Event);
	case Event.What of
		evMouseDown:
			if Event.Buttons <> 1 then Event.What := evNothing;
	end;
end;

procedure TPlat.HandleEvent (Var Event : TEvent);
	procedure Calendar;
	begin
		InsertWindow (New (PCalendarWindow, Init));
	end;

	procedure Calculator;
	begin
		InsertWindow (New (PCalculator, Init));
	end;

begin
	TApplication.HandleEvent (Event);
	case Event.What of
		evCommand:
			begin
				case Event.Command of
					cmFileOpen		: FileOpen ('*.PLA');
					cmCascade	: Cascade;
					cmTile		: Tile;
					cmCalendar	: Calendar;
					cmCalculator: Calculator;
				else
					Exit;
				end;
				ClearEvent(Event);
			end;
	end;
end;

procedure TPlat.Idle;

	function IsTileable (P : PView) : Boolean; far;
	begin
		IsTileable := P^.Options and ofTileable <> 0;
	end;

begin
	TApplication.Idle;
	Clock^.Update;
	Heap^.Update;
	if Desktop^.FirstThat (@IsTileable) <> nil then
		EnableCommands ([cmTile, cmCascade])
	else
		DisableCommands ([cmTile, cmCascade]);
end;

procedure TPlat.InitMenuBar;
	var
		R	: TRect;
	begin
		GetExtent (R);
		R.B.Y := R.A.Y + 1;
		MenuBar := New (PMenuBar, Init (R, NewMenu (
			NewSubMenu	('&'#240'', 0, NewMenu(
				NewItem	('', '', kbNoKey, cmCalendar, 0,
				NewItem	('&', '', kbNoKey, cmCalculator, 0, nil))),
			NewSubMenu	('&', 0, NewMenu(
				NewItem	('&...', 'F3', kbF4, cmFileOpen, 0,
				NewItem	('&...', 'F2', kbF2, cmSave, 0,
				NewLine (
				NewItem	('&室', 'Alt-X', kbAltX, cmQuit, 0, nil))))),
			NewSubMenu	('&', 0, NewMenu(
				NewItem	('&/','Ctrl-F5', kbCtrlF5, cmResize, 0,
				NewItem	('&/', 'F5', kbF5, cmZoom, 0,
				NewItem	('&饥', 'F6', kbF6, cmNext, 0,
				NewItem	('&', 'Alt-F3', kbAltF3, cmClose, 0,
				NewItem	('&९楩', '', kbNoKey, cmTile, 0,
				NewItem	('&᪠', '', kbNoKey, cmCascade, 0, nil))))))), nil))))));
	end;

procedure TPlat.InitStatusLine;
	var
		R	: TRect;
	begin
		GetExtent(R);
		R.A.Y := R.B.Y - 1;
		StatusLine := New (PStatusLine, Init (R,
			NewStatusDef (0, $FFFF,
				NewStatusKey ('~F2~ ', kbF2, cmSave,
				NewStatusKey ('~F3~ ', kbF3, cmFileOpen,
				NewStatusKey ('~Alt-F3~ ', kbAltF3, cmClose,
				NewStatusKey ('~F5~ ', kbF5, cmZoom,
				NewStatusKey ('', kbF10, cmMenu,
				NewStatusKey ('', kbCtrlF5, cmResize, nil)))))), nil)));
	end;

procedure TPlat.OutOfMemory;
	begin
		MessageBox ('筮   믮 .', nil, mfError + mfOkButton);
	end;

procedure TPlat.FileOpen (S : PathStr);
	var
		  D			: PFileDialog;
		  FileName	: PathStr;
	begin
		D := New (PFileDialog, Init (S, ' ', '& 䠩', fdOpenButton + fdHelpButton, 100));
		if ValidView (D) <> nil then
			begin
				if Desktop^.ExecView (D) <> cmCancel then
					begin
						D^.GetFileName (FileName);
						Work (FileName);
					end;
				Dispose (D, Done);
			end;
	end;

procedure TPlat.Work (FileName : PathStr);
	begin
		Desktop^.Insert (ValidView (New (PPltWork, Init (FileName))));
	end;

var
	Demo	: TPlat;
begin
	Demo.Init;
	Demo.DisableCommands ([cmSave, cmNew, cmEdit, cmDouble, cmPrint, cmDel]);
	Demo.Run;
	Demo.Done;
end.
