unit	PltDB;

interface

uses	Objects, DataServ, CharLog, Memory, PltCMD, PltVE;

type

	TPltSorted = (ByNumber, ByDate, BySumma, BySour, ByDest, NoSorted);

	TFData = Object (TObject)
		Code		: Longint;
		CodeX		: Longint;
		MFO			: Longint;
		Schet		: Longint;
		Name		: PString;
		BankN		: PString;
		BankC		: PString;
		constructor	Load (Var S : TStream);
		procedure	Store (Var S : TStream);
		destructor	Done; virtual;
	end;

	PTextBuffer = ^TTextBuffer;
	TTextBuffer = Record
		Length	: Word;
		Text	: array [0..$FFF0] of Char;
	end;

	PPltItem = ^TPltItem;
	TPltItem = Object (TObject)
		Mark		: Boolean;
		Number		: Longint;
		Summa		: Real;
		Date		: TDate;
		Sour		: TFData;
		Dest		: TFData;
		Memo		: PTextBuffer;
		constructor	Init;
		constructor	Load (Var S : TStream);
		procedure	Store (Var S : TStream);
		destructor	Done; virtual;
	end;

	PPltCollection = ^TPltCollection;
	TPltCollection = Object (TSortedCollection)
		SortedBy	: TPltSorted;
		Number		: Longint;
		Modif		: Boolean;
		constructor	Init;
		constructor	Load (Var S : TStream);
		procedure	Store (Var S : TStream);
		function	Compare (P1, P2 : Pointer) : Integer; virtual;
		function	GetItem (Var S : TStream) : Pointer; virtual;
		procedure	PutItem (Var S : TStream; Item : Pointer); virtual;
		procedure	SetSortedStyle (NewSorted : TPltSorted);
		function	GetLine (Var P : TPltItem; N : Integer) : String;
		function	GetForm (N : Integer) : String;
		function	FormLen : Integer;
	end;

implementation
const
	Form	: array [1..36] of String [120] = (
{$I plT4}
	);

{**************************************************************** Routines **}
function LongToSign (L : Longint) : Integer;
{	Inline ($58/$58/$85/$C0/$75/$01/$40);
}
ASSEMBLER;
asm
	MOV		AX, WORD PTR L + 2
	TEST	AX, AX
	JNE		@1
	MOV		AX, WORD PTR L
	TEST	AX, AX
	JE		@1
	MOV		AX, 1
@1:
end;

function RealToSign (R : Real) : Integer;
	begin
		if R > 0.0 then
			RealToSign := 1
		else
			if R = 0.0 then
				RealToSign := 0
			else
				RealToSign := -1;
	end;

{****************************************************************** TFData **}
constructor TFData.Load (Var S : TStream);
	begin
		S.Read (Code, SizeOf (Code) + SizeOf (CodeX) + SizeOf (MFO) + SizeOf (Schet));
		Name := S.ReadStr;
		BankC := S.ReadStr;
		BankN := S.ReadStr;
	end;

procedure TFData.Store (Var S : TStream);
	begin
		S.Write (Code, SizeOf (Code) + SizeOf (CodeX) + SizeOf (MFO) + SizeOf (Schet));
		S.WriteStr (Name);
		S.WriteStr (BankC);
		S.WriteStr (BankN);
	end;

destructor TFData.Done;
	begin
		DisposeStr (Name);
		DisposeStr (BankN);
		DisposeStr (BankC);
	end;

{**************************************************************** TPltItem **}
constructor TPltItem.Init;
	begin
		TObject.Init;
		Sour.Init;
		Dest.Init;
	end;

constructor TPltItem.Load (Var S : TStream);
	Var
		W	: Word;
	begin
		S.Read (Mark, SizeOf (Mark) + SizeOf (Number) + SizeOf (Summa) + SizeOf (Date));
		Sour.Load (S);
		Dest.Load (S);
		S.Read (W, SizeOf (Word));
		GetMem (Memo, W + SizeOf (Word));
		Memo^.Length := W;
		S.Read (Memo^.Text, W);
	end;

procedure TPltItem.Store (Var S : TStream);
	begin
		S.Write (Mark, + SizeOf (Mark) + SizeOf (Number) + SizeOf (Summa) + SizeOf (Date));
		Sour.Store (S);
		Dest.Store (S);
		S.Write (Memo^, SizeOf (Word) + Memo^.Length);
	end;

destructor TPltItem.Done;
	begin
		Sour.Done;
		Dest.Done;
		FreeMem (Memo, Memo^.Length + 2);
	end;

{********************************************************** TPltCollection **}
constructor TPltCollection.Init;
	begin
		TSortedCollection.Init (0, 20);
		Duplicates := True;
		Number := 1;
	end;

constructor TPltCollection.Load (Var S : TStream);
	begin
		TSortedCollection.Load (S);
		S.Read (SortedBy, SizeOf (SortedBy) + SizeOf (Number));
	end;

procedure TPltCollection.Store (Var S : TStream);
	begin
		TSortedCollection.Store (S);
		S.Write (SortedBy, SizeOf (SortedBy) + SizeOf (Number));
	end;

function TPltCollection.Compare (P1, P2 : Pointer) : Integer;
	var
		Item1	: PpltItem absolute P1;
		Item2	: PpltItem absolute P2;
	begin
		case SortedBy of
			ByNumber	: Compare := LongToSign (Item1^.Number - Item2^.Number);
			ByDate		: Compare := LongToSign (DateToInt (Item1^.Date) - DateToInt (Item2^.Date));
			BySumma		: Compare := RealToSign (Item1^.Summa - Item2^.Summa);
			BySour		: Compare := StrCompare (Item1^.Sour.Name^, Item2^.Sour.Name^);
			ByDest		: Compare := StrCompare (Item1^.Dest.Name^, Item2^.Dest.Name^);
		else
			Compare := -1;
		end;
	end;

function TPltCollection.GetItem (Var S : TStream) : Pointer;
	begin
		if LowMemory then
			begin
				S.Status := 1;
				GetItem := NIL;
			end
		else
			GetItem := New (PPltItem, Load (S));
	end;

procedure TPltCollection.PutItem (Var S : TStream; Item : Pointer);
	begin
		PPltItem (Item)^.Store (S);
	end;

procedure TPltCollection.SetSortedStyle (NewSorted : TPltSorted);
	begin
		if NewSorted <> SortedBy then
			begin
				SortedBy := NewSorted;
				MoveSort (Count - 1, 0);
				Modif := True;
			end;
	end;

function TPltCollection.FormLen : Integer;
	begin
		FormLen := 36;
	end;

function TPltCollection.GetForm (N : Integer) : String;
	begin
		GetForm := Form [N];
	end;

function TPltCollection.GetLine (Var P : TPltItem; N : Integer) : String;
	Var
		S : String;

	function Num (L : Longint; N : Byte) : String;
		begin
			if L > 0 then
				begin
					Str (L : N, S);
					ReplaceChar (S, ' ', '0');
					Num := S;
				end
			else
				Num := '';
		end;

	function RNum (R : Real; W : Byte) : String;
		begin
			Str (R : W : 2, S);
			ReplaceChar (S, '.', ',');
			RNum := S;
		end;

	function Nstr (M : String; Max : Integer) : String;
		begin
			if Length (M) <= Max then
				Nstr := M + Space (Max - Length (M))
			else
				Nstr := Copy (M, 1, Max);
		end;

	function GetNStr (N, Max : Integer; Var Buf : TTextBuffer) : String;
		Var
			P	: Word;
			L	: Integer;
		begin
			P := 0;
			L := 0;
			while (N > 1) and (P < Buf.Length) do
				begin
					if Buf.Text [P] < #14 then
						begin
							Inc (P);
							Dec (N);
						end;
					Inc (P);
				end;
			while (N = 1) and (P < Buf.Length) and (L < Max) do
				if Buf.Text [P] < #14 then
					Dec (N)
				else
					begin
						Inc (L);
						S [L] := Buf.Text [P];
						Inc (P);
					end;
			while L < Max do
				begin
					S [L] := ' ';
					Inc (L);
				end;
			S [0] := Char (L);
			GetNStr := S;
		end;

	function GrNStr (N, Max : Integer; M : String; Var Map : array of Byte) : String;
		Var
			P	: Word;
			I, J,
			K, L: Integer;
			function Next (I : Integer) : Integer;
				begin
					repeat
						Inc (I);
					until (I >= Length (M)) or (M [I] = ' ');
					Next := I;
				end;
		begin
			ReplaceChar (M, #13, ' ');
			ReplaceChar (M, #10, ' ');
			UnSpace (M);
			P := 1; I := 0; J := 1; L := 0;
			while (N > 1) and (P <= Length (M)) do
				begin
					P := Next (P);
					if P - I >= Map [L] then
						begin
							Dec (N);
							Inc (L);
							I := J + 1;
						end
					else
						J := P;
				end;
			K := J; P := J;
			while (N = 1) and (P <= Length (M)) do
				begin
					P := Next (P);
					if P - I >= Map [L] then
						begin
							Inc (L);
							Dec (N);
							I := K + 1;
						end
					else
						K := P;
				end;
			S := Copy (M, J, K - J);
			GrNstr := Nstr (S, Max);
		end;

	const
		Map1	: array [0..2] of Byte = (50, 39, 39);
		Map2	: array [0..2] of Byte = (58, 58, 0);
	begin
		case N of
			2	: GetLine := Space (53) + Num (P.Number, 7);
			3	:
				begin
					FormatDate (P.Date, S);
					GetLine := Space (32) + S;
				end;
			5	: GetLine := Space (12) + GrNStr (1, 50, P.Sour.Name^, Map1);
			6	: GetLine := Space (23) + GrNStr (2, 39, P.Sour.Name^, Map1);
			7	: GetLine := Space (16) + Num (P.Sour.Code, 5) + Space (3) + GrNStr (3, 39, P.Sour.Name^, Map1);
			8	: GetLine := Space (65) + Num (P.Sour.Schet, 9);
			9	: GetLine := Nstr (P.Sour.BankN^, 52) + Num (P.Sour.MFO, 9) + Space (15) + RNum (P.Summa, 10);
			10	: GetLine := Space (22) + NStr (P.Sour.BankC^, 22);

			12	: GetLine := Space (12) + GrNStr (1, 50, P.Dest.Name^, Map1);
			13	: GetLine := Space (23) + GrNStr (2, 39, P.Dest.Name^, Map1) + '   ' + Num (P.Dest.Schet, 9);
			14	: GetLine := Space (16) + Num (P.Dest.Code, 5) + Space (3) + GrNStr (3, 39, P.Dest.Name^, Map1);
			16	: GetLine := Nstr (P.Dest.BankN^, 52) + Num (P.Dest.MFO, 9);
			17	: GetLine := Space (22) + NStr (P.Dest.BankC^, 22);

			20	: GetLine := Space (16) +GrNstr (1, 58, ScriptSumma (P.Summa), Map2);
			21	: GetLine := Space (16) + GrNstr (2, 58, ScriptSumma (P.Summa), Map2);
			27..32: GetLine := Space (20) + GetNStr (N - 26, 54, P.Memo^);
		else
			GetLine := '';
		end;
	end;
end.