{$F+}
unit	PltVE;

interface

uses	Objects, Views, Dialogs, Drivers, PltCMD, CharLog, DataServ, Number;

type

	PSummaInput = ^TSummaInput;
	TSummaInput = Object (TInputNumber)
		constructor		Init (Var R : TRect);
		function		DataSize : Word; virtual;
		procedure		GetData (Var R); virtual;
		procedure		SetData (Var R); virtual;
		procedure		HandleEvent (Var Event : TEvent); virtual;
	end;

	PDateView = ^TDateView;
	TDateView = Object (TGroup)
		Day			: PInputLine;
		Month		: PInputLine;
		Year		: PInputLine;
		constructor Init (Var R : TRect);
		constructor	Load (Var S : TStream);
		procedure	Store (Var S : TStream);
		procedure	HandleEvent (Var Event : TEvent); virtual;
		function	DataSize : Word; virtual;
		procedure	GetData (Var Data); virtual;
		procedure	SetData (Var Data); virtual;
		procedure	SelectAll;
	end;

	PSummaScript = ^TSummaScript;
	TSummaScript = Object (TInputLine)
		constructor	Init (Var R : TRect);
		procedure	HandleEvent (Var Event : TEvent); virtual;
		function	DataSize : Word; virtual;
	end;

	function ScriptSumma (R : Real) : String;

implementation

{**************************************************************** Routines **}
type

	TItemsNamesGen = function (I : Byte) : String;

const
	EatName			: array [0..9] of String [6] =
	('', '', '', '', '', '', '', 'ᥬ', 'ᥬ', '');

	Eat_Name		: array [11..19] of String [12] =
	('', '', 'ਭ', 'ୠ', '⭠',
	 '⭠', 'ᥬ', 'ᥬ', '⭠');

	 Eat_x10Name	: array [1..9] of String [10] =
	('', '', 'ਤ', 'ப', '줥', '줥',
	 'ᥬ줥', 'ᥬ줥', 'ﭮ');

	 Eat_x100Name	: array [1..9] of String [9] =
	 ('', '', '', '', '', '', 'ᥬ',
	  'ᥬ', '');

function xSOM (I : Byte) : String;
	begin
		case I of
			2,
			3,
			4	: xSOM := 'ᮬ';
		else
			xSOM := 'ᮬ';
		end;
	end;

function xTYN (I : Byte) : String;
	begin
		case I of
			2,
			3,
			4	: xTYN := '뭠';
		else
			xTYN := '';
		end;
	end;

function x10_3 (I : Byte) : String;
	begin
		case I of
			1	: x10_3 := '';
			2,
			3,
			4	: x10_3 := '';
		else
			x10_3 := '';
		end;
	end;

function x10_6 (I : Byte) : String;
	begin
		case I of
			1	: x10_6 := '';
			2,
			3,
			4	: x10_6 := '';
		else
			x10_6 := '';
		end;
	end;

function x10_9 (I : Byte) : String;
	begin
		case I of
			1	: x10_9 := '';
			2,
			3,
			4	: x10_9 := 'ठ';
		else
			x10_9 := 'म';
		end;
	end;

function IntToString (N : Integer; Names : TItemsNamesGen; As, Is : Boolean) : String; near;
	var
		S	: String;
		I	: Integer;
	begin
		I := N div 100;
		if I > 0 then
			S := Eat_x100Name [I] + ' '
		else
			S := '';
		N := N mod 100;
		if (N > 10) and (N < 20) then
			begin
				S := S + Eat_Name [N] + ' ';
				N := 0;
			end
		else
			begin
				I := N div 10;
				if I > 0 then S := S + Eat_x10Name [I] + ' ';
				I := N mod 10;
				if (I > 0) then
					if Is and ((I = 1) or (I = 2)) then
						if I = 1 then
							S := S + ' '
						else
							S := S + ' '
					else
						S := S + EatName [I] + ' ';
			end;
		if (Length (S) > 0) or As then S := S + Names (N mod 10) + ' ';
		IntToString := S;
	end;

function LongToString (L : Longint; Names : TItemsNamesGen; Sd : Boolean) : String; near;
	var
		S : String;
	begin
		if L > 0 then
			if Sd then
				LongToString :=
					IntToString (L div 1000000000, x10_9, False, False) +
					IntToString ((L mod 1000000000) div 1000000, x10_6, False, False) +
					IntToString ((L mod 1000000) div 1000, x10_3, False, True) +
					IntToString (L mod 1000, Names, True, False)
			else
				begin
					Str (L, S);
					LongToString := S + ' ' + Names (L mod 10);
				end
		else
			LongToString := '';
	end;

function ScriptSumma (R : Real) : String;
	var
		S	: String;
	begin
		if Abs (R) < MaxLongint then
			begin
				R := Trunc (R) + Round (Frac (R) * 100) / 100;
				S := LongToString (Trunc (R), xSOM, True)
					+ LongToString (Round (Frac (R) * 100), xTYN, False);
				S [1] := UpCase (S [1]);
				ScriptSumma := S;
			end
		else
			ScriptSumma := '祭 ...';
	end;

{************************************************************* TSummaInput **}
constructor TSummaInput.Init (Var R : TRect);
	begin
		TInputLine.Init (R, 20);
	end;

function TSummaInput.DataSize : Word;
	begin
		DataSize := SizeOf (Real);
	end;

procedure TSummaInput.SetData (Var R);
	var
		S	: String;
	begin
		Str (Real (R) : 12 : 2, S);
		TInputNumber.SetData (S);
	end;

procedure TSummaInput.GetData (Var R);
	var
		I	: Integer;
		S	: String;
	begin
		S := Data^;
		ReplaceChar (S, ',', '.');
		RemoveLowSpace (S);
		Val (S, Real (R), I);
	end;

procedure TSummaInput.HandleEvent (Var Event : TEvent);
	var
		R, N	: Real;
	begin
		GetData (R);
		TInputNumber.HandleEvent (Event);
		if Event.What = evNothing then
			begin
				GetData (N);
				if R <> N then
					Message (Owner, evBroadCast, cmSummaChange, @Self);
			end;
	end;

{************************************************************ TSummaScript **}
constructor TSummaScript.Init (Var R : TRect);
	begin
		TInputLine.Init (R, 255);
		EventMask := {EventMask or }evBroadCast;
		Options := Options and not ofSelectable;
	end;

function TSummaScript.DataSize : Word;
	begin
		DataSize := 0;
	end;

procedure TSummaScript.HandleEvent (Var Event : TEvent);
	var
		R	: Real;
	begin
		if (Event.What = evBroadCast) and (Event.Command = cmSummaChange) then
			begin
				PSummaInput (Event.InfoPtr)^.GetData (R);
				Data^ := ScriptSumma (R);
				Data^ [1] := UpCase (Data^[1]);
				DrawView;
				ClearEvent (Event);
			end
		else
			TInputLine.HandleEvent (Event);
	end;
{*************************************************************** TDateView **}
constructor TDateView.Init (Var R : TRect);
	var
		S : PStaticText;
		D : TDate;
	begin
		TGroup.Init (R);
		GetExtent (R);
		New (S, Init (R, '    /          /       '));
		S^.GrowMode := gfGrowHiX or gfGrowHiY;
		Insert (S);
		R.Assign (0, 0, 4, 1);
		New (Day, Init (R, 2));
		Insert (Day);
		R.Assign (5, 0, 15, 1);
		New (Month, Init (R, 8));
		with Month^ do Options := Options and not ofEditable;
		Insert (Month);
		R.Assign (16, 0, 22, 1);
		New (Year, Init (R, 4));
		Insert (Year);
		FillChar (D, SizeOf (D), 0);
		SetData (D);
	end;

procedure TDateView.SelectAll;
	begin
		Day^.SelectAll (True);
		Month^.SelectAll (True);
		Year^.SelectAll (True);
	end;

constructor TDateView.Load (Var S : TStream);
	begin
		TGroup.Load (S);
		GetSubViewPtr (S, Day);
		GetSubViewPtr (S, Month);
		GetSubViewPtr (S, Year);
	end;

procedure TDateView.Store (Var S : TStream);
	begin
		TGroup.Store (S);
		PutSubViewPtr (S, Day);
		PutSubViewPtr (S, Month);
		PutSubViewPtr (S, Year);
	end;

function TDateView.DataSize : Word;
	begin
		DataSize := SizeOf (TDate);
	end;

procedure TDateView.SetData (Var Data);
	var
		D : TDate absolute Data;
	begin
		NDate (D);
		Str (D.Day + 1 : -2, Day^.Data^);
		Month^.Data^:= MonthNames [D.Mon];
		Str (D.Year : -4, Year^.Data^);
		SelectAll;
	end;

procedure TDateView.GetData (Var Data);
	begin
		TextToDate (Day^.Data^ + '\' + Month^.Data^ + '\' + Year^.Data^, TDate (Data));
	end;

procedure TDateView.HandleEvent (Var Event : TEvent);
	var
		D : TDate;
	label
		st;
	begin
		if (Event.What = evKeyDown) and (PView (Month) = Current) then
			begin
				GetData (D);
				case Event.KeyCode of
					kbUp, $3920: if D.Mon < 11 then Inc (D.Mon) else D.Mon := 0;
					kbDown: if D.Mon > 0 then Dec (D.Mon) else D.Mon := 11;
				else
					if Event.CharCode < ' ' then goto st;
				end;
				SetData (D);
				ClearEvent (Event);
			end
		else
			st:
			TGroup.HandleEvent (Event);
	end;
end.