{$D+}

unit	OtdView;

interface

uses	Objects, Drivers, Views, ZarInc, DataServ, SmartDra;

type

	PItemList = ^TItemList;
	TItemList = Object (TListViewer)
		Height		: Integer;
		Link		: PListViewer;
		Title		: PString;
		constructor	Init (Var R : TRect; ATitle : String; AHeight : Integer;
						 ALink : PListViewer; HS, VS : PScrollBar);
		constructor Load (Var S : TStream);
		procedure	Store (Var S : TStream);
		procedure	FocusItem (Item : Integer); virtual;
		procedure	Draw; virtual;
		procedure	PrintStr (Var Buffer : TSmartBuffer; Line : Integer);
		destructor	Done; virtual;
	end;

	PMonthList = ^TMonthList;
	TMonthList = Object (TItemList)
		function	GetText (N, M : Integer) : String; virtual;
	end;

	PPersonList = ^TPersonList;
	TPersonList = Object (TItemList)
		Base		: PCollection;
		Undo		: String [50];
		constructor	Init (Var R : TRect; ATitle : String; AHeight : Integer; ALink : PListViewer; HS, VS : PScrollBar);
		constructor Load (Var S : TStream);
		procedure	Store (Var S : TStream);
		function	Pos : Integer;
		procedure	Seek (APos : Integer);
		procedure	HandleEvent (Var Event : TEvent); virtual;
		procedure	FocusItem (Item : Integer); virtual;
		function	GetText (N, M : Integer) : String; virtual;
		procedure	ResetRange;
	end;

implementation

uses	Person, CharLog;

{************************************************************* TPersonList **}
constructor	TPersonList.Init (Var R : TRect; ATitle : String; AHeight : Integer; ALink : PListViewer; HS, VS : PScrollBar);
	begin
		TItemList.Init (R, ATitle, AHeight, ALink, HS, VS);
		if PtrRec (HS).Seg > 0 then HS^.SetRange (0, 50);
	end;

constructor TPersonList.Load (Var S : TStream);
	begin
		TItemList.Load (S);
		S.Read (Undo, SizeOf (Undo));
	end;

procedure TPersonList.Store (Var S : TStream);
	begin
		TItemList.Store (S);
		S.Write (Undo, SizeOf (Undo));
	end;

procedure TPersonList.FocusItem (Item : Integer);
	var
		S : Integer;
	begin
		S := Focused;
		TItemList.FocusItem (Item);
		SetCursor (Cursor.X, Focused - TopItem + Height);
		SetState (sfCursorVis, Focused < Base^.Count);
		if (S <> Focused) and (Focused < Base^.Count) then
			Undo := PPerson (Base^.At (Focused))^.Info.FIO;
	end;

procedure TPersonList.ResetRange;
	begin
		SetRange (Base^.Count);
		DrawView;
	end;

procedure TPersonList.HandleEvent (Var Event : TEvent);
	var
		I : Integer;
	begin
		if (Event.What = evKeyDown) and (Focused < Base^.Count) then
			begin
				if (Event.CharCode > ^U) and (Options and ofEditable <> 0) then
					with PPerson (Base^.At (Focused))^.Info do
						begin
							if Pos > Length (FIO) then FIO := FIO + Space (Pos - Length (FIO));
							if State and sfCursorIns = 0 then
								begin
									if Length (FIO) < 50 then
										System.Insert (Event.CharCode, FIO, Pos + 1);
								end
							else
								FIO [Pos + 1] := Event.CharCode;
							Seek (Pos + 1);
						end
				else
					if Event.CharCode = ^U then
						PPerson (Base^.At (Focused))^.Info.FIO := Undo
					else
						case Event.KeyCode of
							kbLeft:	Seek (Pos - 1);
							kbRight:Seek (Pos + 1);
							kbHome:	Seek (0);
							kbEnd:
								with PPerson (Base^.At (Focused))^.Info do
									begin
										I := Length (FIO);
										while (I > 0) and (FIO [I] <= ' ') do Dec (I);
										Seek (I);
									end;
							kbBack,
							kbDel:
							if Options and ofEditable <> 0 then
								begin
									if Event.KeyCode = kbBack then
										if Pos > 0 then	Seek (Pos - 1) else Exit;
									with PPerson (Base^.At (Focused))^.Info do
										System.Delete (FIO, Pos + 1, 1);
								end;
							kbIns: SetState (sfCursorIns, State and sfCursorIns = 0);
						else
							TListViewer.HandleEvent (Event);
							Exit;
						end;
				DrawView;
				ClearEvent (Event);
			end
		else
			TListViewer.HandleEvent (Event);
	end;

function TPersonList.Pos;
	begin
		Pos := Cursor.X + HScrollBar^.Value - 1;
	end;

procedure TPersonList.Seek (APos : Integer);
	var
		Delta	: Integer;
		E		: Word;
	begin
		if APos < 0 then APos := 0;
		if APos > 49 then
			begin
				if PtrRec (Owner).Seg > 0 then Owner^.SelectNext (False);
				APos := 49;
			end;
		Delta := HScrollBar^.Value;
		if Delta > APos then Delta := APos;
		if APos - Delta >= Size.X - 2 then Delta := APos - Size.X + 3;
		if Delta <> HScrollBar^.Value then
			begin
				E := EventMask;
				EventMask := evNothing;
				HScrollBar^.SetValue (Delta);
				EventMask := E;
			end;
		SetState (sfCursorVis, Focused < Range);
		SetCursor (APos - Delta + 1, Cursor.Y);
	end;

function TPersonList.GetText (N, M : Integer) : String;
	begin
		GetText := Copy (PPerson (Base^.At (N))^.Info.FIO, M, Size.X - 2);
	end;

{*************************************************************** TItemList **}
constructor	TItemList.Init (Var R : TRect; ATitle : String; AHeight : Integer;
							 ALink : PListViewer; HS, VS : PScrollBar);
	begin
		TListViewer.Init (R, 1, HS, VS);
		GrowMode := gfGrowHiY;
		Link := ALink;
		Title := NewStr (ATitle);
		Height := AHeight;
		if PtrRec (HS).Seg > 0 then HS^.SetRange (0, 50);
		if PtrRec (VS).Seg > 0 then VS^.SetStep (Size.Y - AHeight, 1);
		SetCursor (1, 1);
	end;

constructor TItemList.Load (Var S : TStream);
	begin
		TListViewer.Load (S);
		S.Read (Height, SizeOf (Height));
		GetPeerViewPtr (S, Link);
		Title := S.ReadStr;
	end;

procedure TItemList.Store (Var S : TStream);
	begin
		TListViewer.Store (S);
		S.Write (Height, SizeOf (Height));
		PutPeerViewPtr (S, Link);
		S.WriteStr (Title);
	end;

destructor TItemList.Done;
	begin
		TListViewer.Done;
		DisposeStr (Title);
	end;

procedure TItemList.FocusItem (Item : Integer);
	var
		S			: Integer;
		L			: PListViewer;
		E			: Word;
	begin
		S := TopItem;
		if Item > Range then Item := Range;
		if Item < 0 then Item := 0;
		if S > Item then S := Item;
		if Item - S >= Size.Y - Height then
			S := Item - Size.Y + Height + 1;
		if S <> TopItem then
			begin
				E := EventMask;
				EventMask := evNothing;
				TopItem := S;
				if PtrRec (VScrollBar).Seg > 0 then
					VScrollBar^.SetValue (S);
				EventMask := E;
			end;
		Focused := Item;
		if Focused >= Range then Focused := Range - 1;
		if (PtrRec (Link).Seg > 0) and (Link^.Focused <> Item) then
			begin
				L := Link;
				Link := nil;
				L^.FocusItem (Item);
				Link := L;
			end;
		if Focused < 0 then Focused := 0;
		DrawView;
	end;

procedure TItemList.PrintStr (Var Buffer : TSmartBuffer; Line : Integer);
	var
		I : Integer;
		S : String;
	begin
		if (Line < 0) or (Line > Range + Height) then Exit;
		if Line < Height then
			if Line = 0 then
				begin
					Buffer.AddChar ('');
					Buffer.RepeatChar (Size.X - 2, '');
					Buffer.AddChar ('');
				end
			else
				if Line = Height - 1 then
					begin
						Buffer.AddChar ('');
						Buffer.RepeatChar (Size.X - 2, '');
						Buffer.AddChar ('');
					end
				else
					if Line = (Height - 1) div 2 then
						begin
							Buffer.AddChar ('');
							I := (Size.X - 2 - Length (Title^)) div 2;
							if I < 0 then I := 0;
							Buffer.Pure (I);
							Buffer.AddClip (Title^, 0, Size.X - 2 - I - I);
							I := Size.X - 2 - I - Length (Title^);
							if I < 0 then I := 0;
							Buffer.Pure (I);
							Buffer.AddChar ('');
						end
					else
						begin
							Buffer.AddChar ('');
							Buffer.Pure (Size.X - 2);
							Buffer.AddChar ('');
						end
			else
				if Line - Height < Range then
					begin
						Buffer.AddChar ('');
						S := GetText (Line - Height, 0);
						I := Size.X - 2 - Length (S);
						if I < 0 then I := 0;
						Buffer.AddClip (S, 0, Size.X - 2);
						Buffer.Pure (I);
						Buffer.AddChar ('');
					end
				else
					if Line - Height = Range then
						begin
							Buffer.AddChar ('');
							Buffer.RepeatChar (Size.X - 2, '');
							Buffer.AddChar ('');
						end;
	end;

procedure TItemList.Draw;
	var
		B : TDrawBuffer;
		N, Y, I, S : Integer;
		C : Byte;
		NC, SC, FC, MC, OC : Byte;
	begin
		NC := Lo (GetColor (1));
		SC := Lo (GetColor (4));
		MC := Lo (GetColor (6));
		if PtrRec (Owner).Seg > 0 then OC := Lo (Owner^.GetColor (1));
		if State and (sfSelected + sfActive) = (sfSelected + sfActive) then
			FC := Lo (GetColor (3))
		else
			FC := Lo (GetColor (4));
		N := TopItem;
		if PtrRec (HScrollBar).Seg > 0 then
			S := HScrollBar^.Value + 1
		else
			S := 1;
		for Y := 0 to Size.Y do
			begin
				if Y < Height then
					if Y = 0 then
						begin
							MoveChar (B, '', NC, Size.X);
							WordRec (B [Size.X - 1]).Lo := Ord ('');
							WordRec (B [0]).Lo := Ord ('');
						end
					else
						if Y = Height - 1 then
								begin
									MoveChar (B, '', NC, Size.X);
									WordRec (B [Size.X - 1]).Lo := Ord ('');
									WordRec (B [0]).Lo := Ord ('');
								end
							else
							if Y = (Height - 1) div 2 then
								begin
									MoveChar (B, ' ', NC, Size.X);
									I := (Size.X - Length (Title^)) div 2;
									if I < 1 then I := 1;
									if State and (sfSelected + sfActive) = (sfSelected + sfActive) then
										MoveStr (B [I], Title^, SC)
									else
										MoveStr (B [I], Title^, NC);
									WordRec (B [Size.X - 1]).Lo := Ord ('');
									WordRec (B [Size.X - 1]).Hi := NC;
									WordRec (B [0]).Lo := Ord ('');
							end
							else
								begin
									MoveChar (B, ' ', NC, Size.X);
									WordRec (B [Size.X - 1]).Lo := Ord ('');
									WordRec (B [0]).Lo := Ord ('');
								end
			else
				if N < Range then
					begin
						if N = Focused then
							C := FC
						else
							if N = Marker then
								C := MC
							else
								if IsSelected (N) then
									C := SC
								else
									C := NC;
						MoveChar (B, ' ', C, Size.X);
						MoveStr (B [1], GetText (N, S), C);
						WordRec (B [Size.X - 1]).Lo := Ord ('');
						WordRec (B [Size.X - 1]).Hi := NC;
						WordRec (B [0]).Lo := Ord ('');
						WordRec (B [0]).Hi := NC;
						Inc (N);
					end
				else
					if N = Range then
						begin
							MoveChar (B, '', NC, Size.X);
							WordRec (B [Size.X - 1]).Lo := Ord ('');
							WordRec (B [0]).Lo := Ord ('');
							Inc (N);
						end
					else
						MoveChar (B, ' ', OC, Size.X);
				WriteLine (0, Y, Size.X, 1, B);
			end;
	end;

{************************************************************** TMonthList **}
function TMonthList.GetText (N, M : Integer) : String;
	var
		I : Integer;
	begin
		if (N + 1) mod 4 = 0 then
			begin
				I := (Size.X - 10) div 2;
				GetText := RepeatChar (I, '<') + Char (49 + N div 4) + ' ' + RepeatChar (I, '>');
			end
		else
			GetText := MonthNames [N - N div 4];
	end;

end.