{$D+}

unit	Other;

interface

uses	Objects, Drivers, Dialogs, Views, StdDlg, Dos, ByDos, App, Memory, MsgBox, Menus,
		ZarInc, ZarHlp, WorkMain, OtdView,
		WorkInfo, WorkRez, WorkTab, WorkChisl, WorkFond, WorkBLis, WorkAdd, WorkSub;

procedure SaveDesktop;
procedure LoadMain (Var S : TStream);
procedure RetrieveDesktop;
procedure StoreMain (Var S : TStream);
procedure About;
procedure LoadDesktop (Var S : TStream);
procedure FileOpen (WildCard : PathStr);
procedure OpenBase (Var AName : String);
procedure ChangeDir;
procedure Tile;
procedure Cascade;
procedure Calculator;
procedure Calendar;
procedure RegisterAll;

function MakeMenuBar : PMenuBar;
function MakeStatusLine : PStatusLine;
function CalcHelpName : PathStr;

const
	DskName : String [8] = 'ZAR2.DSK';
	DskKey	: Longint = $1234354;
	DskSig	: String [45] = 'Zar 1.0 desktop file (with CyrVision 2.0).';

implementation

uses	Calc, Cale, Otdel, FileServ;

function CalcHelpName : PathStr;
	var
		Dir		: DirStr;
		Name	: NameStr;
		Ext		: ExtStr;
	begin
		FSplit (ParamStr (0), Dir, Name, Ext);
		CalcHelpName := FSearch ('ZARHLP.HLP', Dir + GetEnv ('PATH'));
	end;

procedure LoadDesktop (Var S : TStream);
	var
		R	: TRect;
		P	: PDeskTop;
	begin
		if S.ReadSignature (DskKey) and DeskTop^.Valid (cmClose) then
			begin
				LoadMain (S);
				New (P, Load (S));
				if (P = nil) or not S.Test then
					DisposeObj (P)
				else
					begin
						Application^.GetExtent (R);
						Inc (R.A.Y, 2); Dec (R.B.Y);
						P^.ChangeBounds (R);
					end;
				with Application^ do
					begin
						if ValidView (P) <> nil then
							begin
								DeskTop^.Free;
								DeskTop := P;
								Insert (P);
							end;
						ReDraw;
					end;
			end;
	end;

procedure RetrieveDesktop;
	var
		S	: TBufStream;
		M	: String;
	begin
		S.Init (DskName, stOpenRead, 1024);
		if LowMemory then
			Application^.OutOfMemory
		else
			if not S.Test then
				MessageBox (#3'   䠩 ﭨ.'#13, nil, mfCancelButton + mfError)
			else
				begin
					LoadDesktop (S);
					if S.Status <> stOk then
						begin
							if S.Status = 2 then
								begin
									dosErase (DskName);
									M :=	#3' ﭨ ᯮ祭'#13+
											#3' 㤥 ᮧ .'#13;
								end
							else
								M := #3'   䠩 ﭨ.'#13;
							MessageBox (M, nil, mfCancelButton + mfError);
						end;
				end;
		S.Done;
	end;

procedure SaveDesktop;
	var
		S : TBufStream;
	begin
		S.Init (DskName, stCreate, 1024);
		if LowMemory then
			Application^.OutOfMemory
		else
			begin
				S.WriteSignature (DskKey);
				StoreMain (S);
				Desktop^.Store (S);
				if not S.Test then
					begin
						MessageBox (#3'  ᮧ 䠩'#13#3+ DskName, nil, mfCancelButton + mfError);
						dosErase (DskName);
					end;
			end;
		S.Done;
	end;

procedure FileOpen (WildCard : PathStr);
	var
		D			: PFileDialog;
		FileName	: PathStr;
	begin
		D := New (PFileDialog, Init (WildCard, '  ଠ樨',
			'&', fdOpenButton + fdHelpButton, 100));
		GetDir (0, FileName);
{		D := New (PFileDialog, Init (WildCard, FileName));}
		D^.HelpCtx := hcFOFileOpenDBox;
		if ValidView(D) <> nil then
			begin
				if Desktop^.ExecView(D) <> cmCancel then
					begin
						D^.GetFileName (FileName);
						OpenBase (FileName);
					end;
				Dispose (D, Done);
			end;
	end;

procedure OpenBase (Var AName : String);
	var
		O	: POtdel;
		P	: Pointer;
	begin
		P := @AName;
		if FileExist (AName) then
			New (O, Init (GenStr (P)))
		else
			if MessageBox ( #3'  䠩 ~%s~ ,'#13+
							#13+
							#3'  ?', @P, mfConfirmation + mfYesNoCancel) = cmYes then
				New (O, Create (AName))
			else
				Exit;
		if O^.Head.Status <> stOk then
			begin
				Dispose (O, Done);
				MessageBox (#3'訡  䠩 ~%s~', @P, mfError+mfOkButton);
			end
		else
			P := New (PWorkMain, Init (O));
	end;

procedure LoadMain (Var S : TStream);
	var
		B : Boolean;
		P : PString;
	begin
		S.Read (B, SizeOf (B));
		if B then
			begin
				P := S.ReadStr;
				OpenBase (P^);
				DisposeStr (P);
			end;
	end;

procedure StoreMain (Var S : TStream);
	var
		B : Boolean;
	begin
		B := PtrRec (PMain).Seg <> 0;
		S.Write (B, SizeOf (B));
		if B then
			PMain^.Otdel^.Store (S);
	end;

procedure About;
	var
		D		: PDialog;
		Control	: PView;
		R		: TRect;
	begin
		R.Assign (0, 0, 50, 11);
		D := New(PDialog, Init(R, ' ⥬'));
		with D^ do
			begin
				Options := Options or ofCentered;
				R.Grow (-1, -1);
				Dec (R.B.Y, 3);
				Insert (New (PStaticText, Init (R,
					#13 +
					^C'~稩 ⮫ 壠~ / CyrVision 2.0'#13 +
					#13 +
					^C'Copyright (C) 1992, 93'#13 +
					#13 +
					^C'by Leonid Youriev')));
				R.Assign(18, 8, 33, 10);
				Insert (New (PButton, Init (R, ' 竨 &?', cmOk, bfDefault)));
			end;
		if ValidView (D) <> nil then
			begin
				Desktop^.ExecView (D);
				Dispose (D, Done);
			end;
	end;

procedure ChangeDir;
	var
		D	: PChDirDialog;
	begin
		D := New (PChDirDialog, Init (cdNormal + cdHelpButton, 101));
		D^.HelpCtx := hcFCChDirDBox;
		if ValidView (D) <> nil then
			begin
				DeskTop^.ExecView (D);
				Dispose (D,Done);
			end;
	end;

procedure Tile; 
	var
		R	: TRect;
	begin
		Desktop^.GetExtent (R);
		Desktop^.Tile (R);
	end;

procedure Cascade; 
	var
		R	: TRect;
	begin
		Desktop^.GetExtent (R);
		Desktop^.Cascade (R);
	end;

procedure Calendar;
	var
		P : PCalendarWindow;
	begin
		P := New (PCalendarWindow, Init);
		P^.HelpCtx := hcCalendar;
		InsertWindow (P);
	end;

procedure Calculator;
	var
		P	: PCalculator;
	begin
		P := New (PCalculator, Init);
		P^.HelpCtx := hcCalculator;
		InsertWindow (P);
	end;

function MakeMenuBar : PMenuBar;
	var
		R	: TRect;
	begin
		Application^.DisableCommands ([cmSave, cmSaveAs, cmClear, cmCopy, cmPaste, cmFind, cmReplace, cmAgain, cmUndo, cmCut]);
		Application^.GetExtent (R);
		Inc (R.A.Y);
		R.B.Y := R.A.Y + 1;
		MakeMenuBar := New (PMenuBar, Init (R, NewMenu (
			NewSubMenu	('&', hcSystem, NewMenu (
				NewItem	('& ⥬', '', kbAltSpace, cmAbout, hcSAbout,
				NewLine	(
				NewItem	('&', '', kbNoKey, cmCalendar, hcSCalendar,
				NewItem	('&', '', kbF9, cmCalculator, hcCalculator, nil))))),
			NewSubMenu	('&', hcFile, NewMenu(
				NewItem	(' &⠫...', '', kbNoKey, cmChDir, hcFChangeDir,
				NewItem	('&࠭ ⮫', '', kbNoKey, cmSaveDesktop, hcOSaveDesktop,
				NewItem	('&⠭ ⮫', '', kbNoKey, cmRetrieveDesktop, hcORestoreDesktop,
				NewLine	(
				NewItem	('&⮭', 'Alt+Esc', kbAltEsc, cmDosShell, hcFDosShell,
				NewItem	('&室', 'Alt+X', kbAltX, cmQuit, hcFExit, nil))))))),
			NewSubMenu	('&', hcWindows, NewMenu(
				NewItem	('&','Ctrl+F5', kbCtrlF5, cmResize, hcWSizeMove,
				NewItem	('&','Ctrl+F5', kbCtrlF5, cmResize, hcWSizeMove,
				NewItem	('&ᯠ', 'F5', kbF5, cmZoom, hcWZoom,
				NewItem	('&饥', 'F6', kbF6, cmNext, hcWNext,
				NewItem	('&।饥', 'Shift+F6', kbShiftF6, cmPrev, hcWNext,
				NewItem	('&', 'Alt+F3', kbAltF3, cmClose, hcWClose,
				NewItem	('&९楩', '', kbNoKey, cmTile, hcWTile,
				NewItem	('&᪠', '', kbNoKey, cmCascade, hcWCascade, nil))))))))),
			NewSubMenu	('&', 0, NewMenu (
				NewItem	('&...', 'F4', kbF4, cmOpenMain, 0,
				NewItem	('&', 'Alt+F4', kbAltF4, cmCloseMain, 0,
				NewSubMenu ('&', 0, NewMenu (
					NewItem (' &᫥...', '', 0, cmSVadd, 0,
					NewItem (' &㤥ঠ...', '', 0, cmSVsub, 0,
					NewItem ('& ௫...', '', 0, cmFond, 0,
					NewItem ('&᫥...', '', 0, cmChisl, 0,
					NewItem ('&쭨 ', '', 0, cmBList, 0, nil)))))),
				NewLine	(
				NewItem	('&ଠ', '', kbNoKey, cmWorkInfo, 0,
				NewItem	('&', '', kbNoKey, cmWorkTabel, 0,
				NewItem	('&', '', kbNoKey, cmWorkRez, 0, nil)))))))), nil)))))));
		{	NewSubMenu	('&', 0, NewMenu (
				NewItem	('&', 'Ctrl+Ins', kbCtrlIns, cmCopy, 0,
				NewItem	('&⠢', 'Shift+Ins', kbShiftIns, cmPaste, 0,
				NewItem	('&१', 'Shist+Del', kbShiftDel, cmCut, 0,
				NewItem	(cDelStr, 'Ctrl+Del', kbCtrlDel, cmDel, 0,
				NewLine	(
				NewItem	('&...', 'F7', kbF7, cmFind, 0,
				NewItem	('&...', 'Shift+F7', kbShiftF7, cmReplace, 0,
				NewItem	('&', 'Ctrl+F7', kbCtrlF7, cmAgain, 0,
				NewLine	(
				NewItem	(cNewStr, 'Alt+Ins', kbAltIns, cmNew, 0,
				NewItem	('&', 'F2', kbF2, cmSave, 0,
				NewItem	(' &...', 'Shift+F2', kbShiftF2, cmSaveAs, 0,
				NewItem	('&', 'Alt+Del', kbAltDel, cmClear, 0,
				NewLine	(
				NewItem	('&', '', kbNoKey, cmClip, 0,
				NewItem	(cEdtStr, 'Alt+', kbAltEnter, cmEdit, 0, nil))))))))))))))))),
			NewSubMenu	('&⮫', hcOptions, NewMenu(
				NewItem	('&࠭ ⮫', '', kbNoKey, cmSaveDesktop, hcOSaveDesktop,
				NewItem	('&⠭ ⮫', '', kbNoKey, cmRetrieveDesktop, hcORestoreDesktop, nil))), nil)))))))));
			}
	end;

function MakeStatusLine : PStatusLine;
	var
		R	: TRect;
	begin
		Application^.GetExtent (R);
		R.A.Y := R.B.Y - 1;
		MakeStatusLine := New (PStatusLine, Init (R,
			NewStatusDef (0, $FFFF,
				NewStatusKey ('~F1~ ', kbF1, cmHelp,
				NewStatusKey ('~F10~ ', kbF10, cmMenu,
				NewStatusKey ('~Alt-X~ 室', kbAltX, cmQuit,
				NewStatusKey ('~F2~ ', kbF2, cmSave,
				NewStatusKey ('', kbShiftF2, cmSaveAs,
				NewStatusKey ('', kbAltEsc, cmDosShell,
				NewStatusKey ('', kbF9, cmCalculator,
				NewStatusKey ('', kbCtrlF5, cmResize,
				NewStatusKey ('', kbF5, cmZoom,
				NewStatusKey ('', kbAltF3, cmClose, nil)))))))))), nil)));
	end;

const

	RWorkMain : TStreamRec = (
		ObjType	: 5000;
		VmtLink	: Ofs (TypeOf (TWorkMain)^);
		Load	: @TWorkMain.Load;
		Store	: @TWorkMain.Store);

	RPersonList : TStreamRec = (
		ObjType	: 5001;
		VmtLink	: Ofs (TypeOf (TPersonList)^);
		Load	: @TPersonList.Load;
		Store	: @TPersonList.Store);

	RMonthList : TStreamRec = (
		ObjType	: 5002;
		VmtLink	: Ofs (TypeOf (TMonthList)^);
		Load	: @TMonthList.Load;
		Store	: @TMonthList.Store);

	RAddList : TStreamRec = (
		ObjType	: 5003;
		VmtLink	: Ofs (TypeOf (TAddList)^);
		Load	: @TAddList.Load;
		Store	: @TAddList.Store);

	ROtherInfo : TStreamRec = (
		ObjType	: 5005;
		VmtLink	: Ofs (TypeOf (TOtherInfo)^);
		Load	: @TOtherInfo.Load;
		Store	: @TOtherInfo.Store);

	RWorkInfo : TStreamRec = (
		ObjType	: 5006;
		VmtLink	: Ofs (TypeOf (TWorkInfo)^);
		Load	: @TWorkInfo.Load;
		Store	: @TWorkInfo.Store);

	RWorkTabel : TStreamRec = (
		ObjType	: 5007;
		VmtLink	: Ofs (TypeOf (TWorkTabel)^);
		Load	: @TWorkTabel.Load;
		Store	: @TWorkTabel.Store);

	RWorkRez : TStreamRec = (
		ObjType	: 5008;
		VmtLink	: Ofs (TypeOf (TWorkRez)^);
		Load	: @TWorkRez.Load;
		Store	: @TWorkRez.Store);

	RWorkFond : TStreamRec = (
		ObjType	: 5009;
		VmtLink	: Ofs (TypeOf (TWorkFond)^);
		Load	: @TWorkFond.Load;
		Store	: @TWorkFond.Store);

	RWorkChisl : TStreamRec = (
		ObjType	: 5010;
		VmtLink	: Ofs (TypeOf (TWorkChisl)^);
		Load	: @TWorkChisl.Load;
		Store	: @TWorkChisl.Store);

	RWorkBList : TStreamRec = (
		ObjType	: 5011;
		VmtLink	: Ofs (TypeOf (TWorkBList)^);
		Load	: @TWorkBList.Load;
		Store	: @TWorkBList.Store);

	RWorkSVsub : TStreamRec = (
		ObjType	: 5012;
		VmtLink	: Ofs (TypeOf (TWorkSVsub)^);
		Load	: @TWorkSVsub.Load;
		Store	: @TWorkSVsub.Store);

	RWorkSVadd : TStreamRec = (
		ObjType	: 5013;
		VmtLink	: Ofs (TypeOf (TWorkSVadd)^);
		Load	: @TWorkSVadd.Load;
		Store	: @TWorkSVadd.Store);

procedure RegisterAll;
	begin
		RegisterType (RWorkMain);
		RegisterType (RPersonList);
		RegisterType (RMonthList);
		RegisterType (RAddList);
		RegisterType (ROtherInfo);
		RegisterType (RWorkInfo);
		RegisterType (RWorkTabel);
		RegisterType (RWorkRez);
		RegisterType (RWorkFond);
		RegisterType (RWorkChisl);
		RegisterType (RWorkBList);
		RegisterType (RWorkSVsub);
		RegisterType (RWorkSVadd);
	end;
end.
