{1/2 Copyright by Leonid Youriev 1992, 93}
{$I VERSION}

unit App;

interface

uses	Objects, Drivers, HistList, Views, Menus, Dialogs,
		Recode, Resident, Styles, AniMouse;

{$I VERSION}

const

{ Standard application help contexts }
	hcNew			= $FF01;
	hcOpen			= $FF02;
	hcSave			= $FF03;
	hcSaveAs		= $FF04;
	hcSaveAll		= $FF05;
	hcChangeDir		= $FF06;
	hcDosShell		= $FF07;
	hcExit			= $FF08;

	hcUndo			= $FF10;
	hcCut			= $FF11;
	hcCopy			= $FF12;
	hcPaste			= $FF13;
	hcClear			= $FF14;

	hcTile			= $FF20;
	hcCascade		= $FF21;
	hcCloseAll		= $FF22;
	hcResize		= $FF23;
	hcZoom			= $FF24;
	hcNext			= $FF25;
	hcPrev			= $FF26;
	hcClose			= $FF27;

type

	TKeyBoardMaps		= (kmCyrPC, kmIBMPC, kmEC1840, kmExample);

{ TBackground Object }
	PBackground = ^TBackground;
	TBackground = Object (TView)
		Pattern		: Char;
		constructor	Init (var Bounds : TRect; APattern : Char);
		constructor	Load (var S : TStream);
		procedure	Draw; virtual;
		procedure	Store (var S : TStream);
	end;

{ TDeskTop Object }
	PDeskTop = ^TDeskTop;
	TDeskTop = Object (TGroup)
		Background		: PBackground;
		WinNumberUses	: set of 0..9;
		TileColumnsFirst: Boolean;
		constructor		Init (var Bounds : TRect);
		constructor		Load (var S : TStream);
		procedure		Store (var S : TStream);
		procedure		Cascade (var R : TRect);
		procedure		HandleEvent (var Event : TEvent); virtual;
		procedure		InitBackground; virtual;
		procedure		Tile (var R : TRect);
		procedure		InsertBefore (P, T : PView); virtual;
		procedure		TileError; virtual;
		function		WinNum : Integer;
		procedure		Delete (P : PView); virtual;
	end;

{ TProgram Object									}
	{ Palette layout								}
	{ 1		= TBackground							}
	{ 2- 7	= TMenuView and TStatusLine				}
	{ 8-15	= TWindow (Blue)						}
	{ 16-23	= TWindow (Cyan)						}
	{ 24-31	= TWindow (Gray)						}
	{ 32-63	= TDialog								}

	PProgram = ^TProgram;
	TProgram = Object (TGroup)
		constructor	Init;
		destructor	Done; virtual;
		procedure	GetEvent (var Event : TEvent); virtual;
		procedure	HandleEvent (var Event : TEvent); virtual;
		procedure	Idle; virtual;
		procedure	InitDeskTop; virtual;
		procedure	InitMenuBar; virtual;
		procedure	InitScreen; virtual;
		procedure	InitStatusLine; virtual;
		function	InitMouseAnimator (AGraph : Boolean) : Pointer; virtual;
		procedure	OutOfMemory; virtual;
		procedure	Run; virtual;
		function	CanMoveFocus : Boolean;
	private
		WantReDraw	: Boolean;
	end;

{ TApplication Object }
	PApplication = ^TApplication;
	TApplication = Object (TProgram)
		constructor	Init;
		destructor	Done; virtual;
		procedure	Cascade;
		procedure	DosShell (Name,  Path : String; Command, Mst : Boolean);
		procedure	GetTileRect (var R : TRect); virtual;
		procedure	HandleEvent (var Event : TEvent); virtual;
		procedure	Tile;
		procedure	WriteShellMsg; virtual;
	end;

{ Standard menus and status lines }
function StdStatusKeys (Next : PStatusItem) : PStatusItem;
function StdFileMenuItems (Next : PMenuItem) : PMenuItem;
function StdEditMenuItems (Next : PMenuItem) : PMenuItem;
function StdWindowMenuItems (Next : PMenuItem) : PMenuItem;

{Application routines}
function ExecuteDialog (P : PDialog; Data : Pointer) : Word;
function InsertWindow (P : PWindow) : PWindow;
procedure SetButtonShadowStyle (The : TButtonShadowStyles);
procedure SetMenuFrameStyle (The : TMenuFrameStyles);
procedure SetScreenStyle (The : TCharCodeStyles; Alfa : Boolean);
procedure SetKeyBoardStyle (The : TCharCodeStyles; Map : TKeyBoardMaps; Alfa : Boolean);
procedure SetMouseStyle (AGraph : Boolean);
procedure SetScreenMode (Mode : Word);
function ValidView (P : PView) : PView;
procedure WantReDraw;

{ App registration procedure }
procedure RegisterApp;

const

{ Public variables }
	Application	: PProgram = NIL;
	DeskTop		: PDeskTop = NIL;
	StatusLine	: PStatusLine = NIL;
	MenuBar		: PMenuView = NIL;

	Config			: record
		ScreenCODE		: TCharCodeStyles;
		ScreenALFA		: Boolean;
		KeyBoardCODE	: TCharCodeStyles;
		KeyBoardALFA	: Boolean;
		KeyBoardMap		: TKeyBoardMaps;
		ShowMarkers		: Boolean;
		Blink			: Boolean
{$IFDEF SupportMasterFont}
		;MasterFont		: Boolean
{$IFDEF SupportMasterMouse}
		;MouseGraph		: Boolean;
{$ENDIF}
{$ENDIF}
	end = (
		ScreenCODE		: csAlt;
		ScreenALFA		: True;
		KeyBoardCODE	: csAlt;
		KeyBoardALFA	: False;
		KeyBoardMap		: kmCyrPC;
		ShowMarkers		: False;
		Blink			: False
{$IFDEF SupportMasterFont}
		;MasterFont		: True
{$IFDEF SupportMasterMouse}
		;MouseGraph		: True
{$ENDIF}
{$ENDIF});

{ Stream registration records }
	RBackground	: TStreamRec = (
		ObjType	: 30;
		VmtLink	: Ofs (TypeOf (TBackground)^);
		Load	: @TBackground.Load;
		Store	: @TBackground.Store);

	RDeskTop	: TStreamRec = (
		ObjType	: 31;
		VmtLink	: Ofs (TypeOf (TDeskTop)^);
		Load	: @TDeskTop.Load;
		Store	: @TDeskTop.Store);

implementation
{$IFDEF UseDark}
uses	Dos, LCR, CharLog, MsgBox, Memory, Quick, Sysvars, Dark;
{$ELSE}
uses	Dos, LCR, CharLog, MsgBox, Memory, Quick, Sysvars;
{$ENDIF}

{************************************************************* TBackground **}
constructor TBackground.Init (var Bounds : TRect; APattern : Char);
begin
	TView.Init (Bounds);
	GrowMode := gfGrowHiX + gfGrowHiY;
	Pattern := APattern;
end;

constructor TBackground.Load (var S : TStream);
begin
	TView.Load (S);
	S.Read (Pattern, SizeOf (Pattern));
end;

procedure TBackground.Draw;
var
	B	: TDrawBuffer;
begin
	MoveChar (B, Pattern, Lo (GetColor (0)), Size.X);
	WriteLine (0, 0, Size.X, Size.Y, B);
end;

procedure TBackground.Store (var S : TStream);
begin
	TView.Store (S);
	S.Write (Pattern, SizeOf (Pattern));
end;

{********************************************************* TDeskTop Object **}
constructor TDeskTop.Init (var Bounds : TRect);
begin
	TGroup.Init (Bounds);
	GrowMode := gfGrowHiX + gfGrowHiY;
	InitBackground;
	Insert (Background);
end;

function Tileable (P : PView) : Boolean;
begin
	Tileable := (P^.Options and ofTileable <> 0) and (P^.State and sfVisible <> 0);
end;

constructor TDeskTop.Load (var S : TStream);
begin
	TGroup.Load (S);
	GetSubViewPtr (S, Background);
	S.Read (WinNumberUses, SizeOf (WinNumberUses) + SizeOf(TileColumnsFirst));
end;

procedure TDeskTop.Store (var S : TStream);
begin
	TGroup.Store (S);
	PutSubViewPtr (S, Background);
	S.Write (WinNumberUses, SizeOf (WinNumberUses) + SizeOf (TileColumnsFirst));
end;

procedure TDeskTop.Delete (P : PView);
var
	W	: Integer;
begin
	W := PWindow (P)^.Number;
	TGroup.Delete (P);
	if (W >= 0) and (W < 10) then
		WinNumberUses := WinNumberUses - [W];
end;

procedure TDeskTop.InsertBefore (P, T : PView);
var
	W	: Integer;
begin
	W := PWindow (P)^.Number;
	if (W > 0) and (W < 10) then
		WinNumberUses := WinNumberUses + [W];
	TGroup.InsertBefore (P, T);
end;

function TDeskTop.WinNum;
var
	W	: Integer;
begin
	WinNum := -1;
	for W := 0 to 9 do
		if not (W in WinNumberUses) then
			begin
				WinNum := W;
				Exit;
			end;
end;

procedure TDeskTop.Cascade (var R : TRect);
var
	CascadeNum	: Integer;
	LastView	: PView;
	Min,
	Max			: TPoint;

	procedure DoCount (P : PView); far;
	begin
		if Tileable (P) then
			begin
				Inc (CascadeNum);
				LastView := P;
			end;
	end;

	procedure DoCascade (P : PView); far;
	var
		NR	: TRect;
	begin
		if Tileable (P) and (CascadeNum >= 0) then
			begin
				NR.Copy (R);
				Inc (NR.A.X, CascadeNum);
				Inc (NR.A.Y, CascadeNum);
				P^.Locate (NR);
				Dec (CascadeNum);
			end;
	end;

begin
	CascadeNum := 0;
	ForEach (@DoCount);
	if CascadeNum > 0 then
		begin
			LastView^.SizeLimits (Min, Max);
			if (Min.X > R.B.X - R.A.X - CascadeNum) or (Min.Y > R.B.Y - R.A.Y - CascadeNum) then
				TileError
			else
				begin
					Dec (CascadeNum);
					Lock;
					ForEach (@DoCascade);
					Unlock;
				end;
		end;
end;

procedure TDeskTop.HandleEvent (var Event : TEvent);
	procedure DoShow (P : PView); far;
		begin
			P^.Show;
		end;

begin
	TGroup.HandleEvent (Event);
	if Event.What = evCommand then
		begin
			case Event.Command of
				cmNext	: FocusNext (False);
				cmPrev	: if Valid (cmReleasedFocus) then Current^.PutInFrontOf (Background);
				cmHide	: Current^.Hide;
				cmShow	: ForEach (@DoShow);
			else
				Exit;
			end;
			ClearEvent (Event);
		end;
end;

procedure TDeskTop.InitBackground;
var
	R	: TRect;
begin
	GetExtent (R);
{$IFDEF SupportMasterMouse}
	New (Background, Init (R, ' '));
{$ELSE}
	New (Background, Init (R, #176));
{$ENDIF}
end;

function ISqr (X : Integer) : Integer; assembler;
asm
	MOV		CX, X
	MOV		BX, 0
@1:	INC		BX
	MOV		AX,	BX
	IMUL	AX
	CMP		AX,	CX
	JLE		@1
	MOV		AX, BX
	DEC		AX
end;

procedure MostEqualDivisors (N : Integer; var X, Y : Integer; FavorY : Boolean);
var
	I : Integer;
begin
	I := ISqr (N);
	if ((N mod I) <> 0) and ((N mod (I + 1)) = 0) then Inc (I);
	if I < (N div I) then I := N div I;
	if FavorY then
		begin
			X := N div I;
			Y := I;
		end
	else
		begin
			Y := N div I;
			X := I;
		end;
end;

procedure TDeskTop.Tile (var R : TRect);
var
	NumCols,
	NumRows,
	NumTileable,
	LeftOver,
	TileNum		: Integer;

	procedure DoCountTileable (P : PView); far;
	begin
		if Tileable (P) then Inc (NumTileable);
	end;

	function DividerLoc (Lo, Hi, Num, Pos : Integer) : Integer;
	begin
		DividerLoc := LongDiv (LongMul (Hi - Lo, Pos), Num) + Lo;
	end;

	procedure CalcTileRect (Pos : Integer; var NR : TRect);
	var
		X,
		Y,
		D	: Integer;
	begin
		D := (NumCols - LeftOver) * NumRows;
		if Pos < D then
			begin
				X := Pos div NumRows;
				Y := Pos mod NumRows;
			end
		else
			begin
				X := (Pos - D) div (NumRows + 1) + (NumCols - LeftOver);
				Y := (Pos - D) mod (NumRows + 1);
			end;
		NR.A.X := DividerLoc (R.A.X, R.B.X, NumCols, X);
		NR.B.X := DividerLoc (R.A.X, R.B.X, NumCols, X + 1);
		if Pos >= D then
			begin
				NR.A.Y := DividerLoc (R.A.Y, R.B.Y, NumRows + 1, Y);
				NR.B.Y := DividerLoc (R.A.Y, R.B.Y, NumRows + 1, Y + 1);
			end
		else
			begin
				NR.A.Y := DividerLoc (R.A.Y, R.B.Y, NumRows, Y);
				NR.B.Y := DividerLoc (R.A.Y, R.B.Y, NumRows, Y + 1);
			end;
	end;

	procedure DoTile (P : PView); far;
	var
		R	: TRect;
	begin
		if Tileable (P) then
			begin
				CalcTileRect (TileNum, R);
				P^.Locate (R);
				Dec (TileNum);
			end;
	end;

begin
	NumTileable := 0;
	ForEach (@DoCountTileable);
	if NumTileable > 0 then
		begin
			MostEqualDivisors (NumTileable, NumCols, NumRows, not TileColumnsFirst);
			if ((R.B.X - R.A.X) div NumCols = 0) or ((R.B.Y - R.A.Y) div NumRows = 0) then
				TileError
			else
				begin
					LeftOver := NumTileable mod NumCols;
					TileNum := NumTileable - 1;
					Lock;
					ForEach (@DoTile);
					Unlock;
				end;
		end;
end;

procedure TDesktop.TileError;
begin
end;

{**************************************************************** TProgram **}
constructor TProgram.Init;
var
	R	: TRect;
begin
{$IFDEF UseDark}
	InitDark;
{$ENDIF}
	Application := @Self;
	R.Assign (0, 0, DevicesInfo.ScreenWidth, DevicesInfo.ScreenHeight);
	TGroup.Init (R);
	InitScreen;
	State := sfVisible + sfSelected + sfFocused + sfExposed;
	SetState (sfModal, True);
	Options := 0;
	Buffer := DevicesInfo.ScreenBuffer;
	SetButtonShadowStyle (TButtonShadowStyles (Config.ShowMarkers));
	SetMenuFrameStyle (MenuFrameStyle);
	InitDeskTop;
	InitStatusLine;
	InitMenuBar;
	Insert (DeskTop);
	Insert (StatusLine);
	Insert (MenuBar);
end;

destructor TProgram.Done;
begin
	DisposeObj (DeskTop);
	DisposeObj (MenuBar);
	DisposeObj (StatusLine);
	Application := NIL;
end;

procedure WantReDraw;
begin
	Application^.WantReDraw := True;
end;

procedure SetButtonShadowStyle (The : TButtonShadowStyles);
begin
	case The of
		bsDefault	: ButtonShadowChars := BuShadowStyleDef;
		bsNone		: ButtonShadowChars := BuShadowNone;
		bsStyleI	: ButtonShadowChars := BuShadowStyleI;
		bsStyleII	: ButtonShadowChars := BuShadowStyleII;
		bsStyleIII	: ButtonShadowChars := BuShadowStyleIII;
		bsStyleVI	: ButtonShadowChars := BuShadowStyleVI;
	end;
	WantReDraw;
end;

procedure SetMenuFrameStyle (The : TMenuFrameStyles);
begin
	MenuFrameStyle := The;
	PartFromStr (MenuFrameChars,
		msMSstr +
		msFreestr +
		msDFreestr +
		msOutstr +
		msDefstr +
		msBoldstr +
		msBoldINstr +
		msBoldOUTstr +
		msBoldBEstr,
		SizeOf (MenuFrameChars), Ord (The));
	WantReDraw;
end;

procedure SetScreenStyle (The : TCharCodeStyles; Alfa : Boolean);
var
	M	: Boolean;
begin
	HideMouse;
	Alfa := Alfa and (The <> csIBMPC);
	Config.ScreenCODE := The;
	Config.ScreenALFA := Alfa;
{$IFDEF SupportMasterFont}
	M := Config.MasterFont;
	SetMaster (False);
{$ENDIF}
	asm
		MOV		DI, OFFSET XlatScreen
		PUSH	DS
		POP		ES
		CLD
		MOV		CX, 256
		XOR		AX, AX
	@1:	STOSB
		INC		AX
		LOOP	@1
	end;
	if Alfa then
		asm
			PUSH	DS
			MOV		DI, OFFSET XlatScreen + 127
			PUSH	DI
			CALL	AlfaStr
		end;
	SCOD (XlatScreen [128], 128, csALT, The);
{$IFDEF SupportMasterFont}
	SetMasterCodeStyle (The);
	SetMaster (M);
{$ENDIF}
	if The = csMIC then
		asm
			CALL	@1
			DB		'', ''
			DB		'', ''
			DB		'', ''
			DB		'', ''
			DB		'', ''
			DB		'', ''
			DB		'', ''
			DB		'', ''
			DB		'', ''
			DB		'', ''
			DB		'', ''
			DB		'', ''
			DB		'', ''
			DB		'', ''
			DB		'', ''
			DB		'', ''
			DB		'', ''
			DB		'', ''
			DW		0
		@1:	POP		SI
			XOR		BX, BX
		@2: SEGCS	LODSW
			MOV		BL, AH
			MOV		AH, BYTE PTR XlatScreen [BX]
			MOV		BL, AL
			MOV		BYTE PTR XlatScreen [BX], AH
			TEST	AL, AL
			JNE		@2
		end;
	Application^.ReDraw;
	ShowMouse;
end;

procedure SetKeyBoardStyle (The : TCharCodeStyles; Map : TKeyBoardMaps; Alfa : Boolean);
var
	I	: Integer;
begin
	Config.KeyBoardALFA := Alfa;
	Config.KeyBoardCODE := The;
	Config.KeyBoardMap := Map;
	asm
		MOV		DI, OFFSET XlatKeyBoard
		PUSH	DS
		POP		ES
		CLD
		MOV		CX, 128
		MOV		AX, CX
	@1:	STOSB
		INC		AX
		LOOP	@1
	end;
	SCOD (XlatKeyBoard, 128, The, csALT);
	if Alfa then
		for I := 0 to 127 do
			XlatKeyBoard [I] := AlfaCase (XlatKeyBoard [I]);
	case Map of						{              }
		kmCyrPC		: KeyBoardMap := 'F'+','+'DULT;PBQRKVYJGHCNEA[WXIO]SM'+#39+'.Z';
		kmEC1840	: KeyBoardMap := 'F'+#39+'DULT[PBQRKVYJGHCNEA{WXIO}SM'+']'+'~Z';
		kmExample	: KeyBoardMap := 'A'+'B'+'WGDEVZIJKLMNOPRSTUFHC^[]_Y '+'|'+'@Q';
		kmIBMPC		: Clear (KeyBoardMap, SizeOf (KeyBoardMap));
	end;
end;

procedure SetMouseStyle (AGraph : Boolean);
begin
	HideMouse;
	DisposeObj (MouseAnimator);
	AdjustMouse (AGraph);
	MouseAnimator := Application^.InitMouseAnimator (AGraph);
	ShowMouse;
end;

function TProgram.CanMoveFocus : Boolean;
begin
	CanMoveFocus := Desktop^.Valid (cmReleasedFocus);
end;

function ExecuteDialog (P : PDialog; Data : Pointer) : Word;
var
	C	: Word;
begin
	ExecuteDialog := cmCancel;
	if Seg (ValidView (P)^) <> 0 then
		begin
			if PtrRec (Data).Seg <> 0 then P^.SetData (Data^);
			C := Desktop^.ExecView (P);
			if (C <> cmCancel) and (PtrRec (Data).Seg <> 0) then P^.GetData (Data^);
			P^.Free;
			ExecuteDialog := C;
		end;
end;

procedure TProgram.GetEvent (var Event : TEvent);
var
	R	: TRect;

	function ContainsMouse (P : PView) : Boolean; far;
	begin
		ContainsMouse := (P^.State and sfVisible <> 0) and P^.MouseInView (Event.Where);
	end;

begin
	TGroup.GetEvent (Event);
	if Event.What <> evNothing then
		Exit
	else
		begin
			GetMouseEvent (Event);
			if Event.What = evNothing then
				begin
					GetKeyEvent (Event);
					if Event.What = evNothing then
					if ShiftStateCall and $8 <> 0 then
						begin
							ShiftStateCall := ShiftStateCall and not $8;
							if Current <> PView (MenuBar) then
								with Event do
									begin
										What := evCommand;
										Command := cmMenu;
										InfoPtr := @Self;
									end
							else
								with Event do
									begin
										What := evKeyDown;
										Command := kbEsc;
										InfoPtr := @Self;
									end;
						end
					else
{$IFDEF UseDark}
						begin
							_;
							if ISroot then Idle;
						end;
{$ELSE}
						Idle;
{$ENDIF}
				end;
		end;
		if PtrRec (StatusLine).Seg <> 0 then
			if (Event.What and evKeyDown <> 0)
			or (Event.What and evMouseDown <> 0)
			and (FirstThat (@ContainsMouse) = PView (StatusLine)) then
				StatusLine^.HandleEvent (Event);
end;

procedure TProgram.HandleEvent (var Event : TEvent);
var
	I	: Word;
	C	: Char;
begin
	if Event.What = evKeyDown then
		begin
			C := GetAltChar (Event.KeyCode);
			if (C >= '1') and (C <= '9') then
				if Seg (Message (DeskTop, evBroadCast, cmSelectWindowNum, Pointer (Byte (C) - $30))^) <> 0 then
					ClearEvent (Event);
		end;
	TGroup.HandleEvent (Event);
	if Event.What = evCommand then
		if Event.Command = cmQuit then
			begin
				EndModal (cmQuit);
				ClearEvent (Event);
			end;
end;

procedure TProgram.Idle;
const
	PredHelpCtx	: Word = $FFFF;
begin
	if WantReDraw then
		begin
			ReDraw;
			WantReDraw := False;
		end;
	if PtrRec (StatusLine).Seg <> 0 then
	StatusLine^.Update;
	if CommandSetChanged then
		begin
			Message (@Self, evBroadcast, cmCommandSetChanged, NIL);
			CommandSetChanged := False;
		end;
	if PredHelpCtx <> GetHelpCtx then
		begin
			PredHelpCtx := GetHelpCtx;
			Message (@Self, evBroadCast, cmHelpCtxChanged, Pointer (PredHelpCtx));
		end;
end;

procedure TProgram.InitDeskTop;
var
	R	: TRect;
begin
	GetExtent (R);
	Inc (R.A.Y);
	Dec (R.B.Y);
	New (DeskTop, Init (R));
end;

procedure TProgram.InitMenuBar;
var
	R	: TRect;
begin
	GetExtent (R);
	R.B.Y := R.A.Y + 1;
	MenuBar := New (PMenuBar, Init (R, NIL));
end;

procedure TProgram.InitScreen;
begin
	if Lo (DevicesInfo.ScreenMode) <> smMono then
		begin
			Config.ShowMarkers := False;
			ShadowSize.X := 1 + Ord (DevicesInfo.ScreenMode and smFont8x8 = 0);
			ShadowSize.Y := 1;
{			if Lo (DevicesInfo.ScreenMode) = smBW80 then
				AppPalette := CBlackWhite
			else
}				AppPalette := CColor;
		end
	else
		begin
			ShadowSize.X := 0;
			ShadowSize.Y := 0;
			Config.ShowMarkers := True;
{			AppPalette := CMonochrome;
}		end;
	SetScreenStyle (Config.ScreenCODE, Config.ScreenALFA);
	SetKeyBoardStyle (Config.KeyBoardCODE, Config.KeyBoardMap, Config.KeyBoardAlfa);
{$IFDEF SupportMasterFont}
{$IFDEF AutoMasterFont}
	SetMouseStyle (True{Config.MouseGraph and DevicesInfo.Masterable});
	SetMaster (True{DevicesInfo.Masterable});
{$ELSE}
	SetMouseStyle (False);
	SetMaster (False);
{$ENDIF}
{$ENDIF}
	PartFromStr (SpecialChars, SCharsColor + SCharsMono, SizeOf (SpecialChars), Ord (Config.ShowMarkers));
	SetCurFont (0, 0);
	SetBlink (Config.Blink);
end;

procedure TProgram.InitStatusLine;
var
	R	: TRect;
begin
	GetExtent (R);
	R.A.Y := R.B.Y - 1;
	New (StatusLine, Init (R,
		NewStatusDef (0, $FFFF,
			NewStatusKey ('~Alt-X~ 室', kbAltX, cmQuit,
			StdStatusKeys (NIL)), NIL)));
end;

function InsertWindow (P : PWindow) : PWindow;
begin
	InsertWindow := NIL;
	if Seg (ValidView (P)^) <> 0 then
		if Application^.CanMoveFocus then
			begin
				Desktop^.Insert (P);
				InsertWindow := P;
			end
		else
			P^.Free;
end;

procedure TProgram.OutOfMemory;
begin
end;

procedure TProgram.Run;
begin
	Execute;
end;

procedure SetScreenMode (Mode : Word);
var
	R	: TRect;
begin
	HideMouse;
	SetVideoMode (Mode);
	DoneMemory;
	with Application^ do
		begin
			InitScreen;
			Buffer := DevicesInfo.ScreenBuffer;
			R.Assign (0, 0, DevicesInfo.ScreenWidth, DevicesInfo.ScreenHeight);
			ChangeBounds (R);
		end;
	ShowMouse;
end;

function ValidView (P : PView) : PView;
const
	InApp	: Boolean = False;
begin
	ValidView := NIL;
	if PtrRec (P).Seg <> 0 then
		if LowMemory and not InApp then
			begin
				P^.Free;
				if not InApp then
					begin
						InApp := True;
						Application^.OutOfMemory;
						InApp := False;
					end;
			end
		else
			if not P^.Valid (cmValid) then
				P^.Free
			else
				ValidView := P;
end;

{************************************************************ TApplication **}
constructor TApplication.Init;
begin
	InitMemory;
	InitDrivers;
	InitVideo;
	InitEvents;
	InitSysError;
	InitHistory;
	TProgram.Init;
end;

destructor TApplication.Done;
begin
	TProgram.Done;
	DoneHistory;
	DoneSysError;
	DoneEvents;
	DoneVideo;
	DoneDrivers;
	DoneMemory;
end;

procedure TApplication.Cascade;
var
	R	: TRect;
begin
	GetTileRect (R);
	if PtrRec (Desktop).Seg <> 0 then Desktop^.Cascade (R);
end;

procedure TApplication.DosShell (Name,  Path : String; Command, Mst : Boolean);
var
	S	: PathStr;
	I	: Word;
	M	: Boolean;
	C	: String [1];
	O	: Longint;
	oL,
	oH,
	oC	: Longint;
begin
	LockScr;
	HideMouse;
{$IFDEF SupportMasterFont}
	M := Config.MasterFont;
	if M then SetMaster (False);
{$ENDIF}
	S := FSearch (Path, GetEnv ('PATH'));
	if S = '' then  S := FExpand (Path);
	DoneSysError;
	DoneEvents;
	DoneMemory;
	DoneVideo;
	WriteShellMsg;
	SetMemTop (HeapPtr);
	Exec (S, '');
	I := DosError;
	if I = 0 then LockScr;
	SetMemTop (HeapEnd);
	InitMemory;
	InitVideo;
	InitEvents;
	InitSysError;
{$IFDEF SupportMasterFont}
	if M then SetMaster (True);
{$ENDIF}
	Application^.Redraw;
	ShowMouse;
	LockScr;
	if I <> 0 then
		begin
			UnSpace (Name);
			if IsCirCTone (Name [Length (Name)]) then
				C := ''
			else
				C := '';
			if (I = 2) or (I = 3) then
				S := #3 + Name + ' .'#13#3' ' + Path
			else
				if I = 8 then
					S := #3' ' + Name + C + ' 筮 .'
				else
					S := #3'訡 ᪠ ' + Name + C + #13#3' ' + Path;
			if Command then
				begin
					S := S + #13#3'맢  DOS ?';
					I := mfError or mfYesButton or mfNoButton;
				end
			else
				I := mfError or mfAllButton;
		if (MessageBox (S, NIL, I) = cmYes) and Command then
			DosShell (' ', GetEnv ('COMSPEC'), False, False);
		end;
end;

procedure TApplication.GetTileRect (var R : TRect);
begin
	Desktop^.GetExtent (R);
end;

procedure TApplication.HandleEvent (var Event : TEvent);
begin
	TProgram.HandleEvent (Event);
	case Event.What of
		evCommand	:
			begin
				case Event.Command of
					cmTile		: Tile;
					cmCascade	: Cascade;
				else
					Exit;
				end;
				ClearEvent(Event);
			end;
	end;
end;

procedure TApplication.Tile;
var
	R	: TRect;
begin
	GetTileRect(R);
	if PtrRec (Desktop).Seg <> 0 then Desktop^.Tile (R);
end;

procedure TApplication.WriteShellMsg;
begin
	PrintStr ('ᯮ  EXIT  襭...');
end;

{***************************************** Standard menus and status lines **}
function StdStatusKeys (Next : PStatusItem) : PStatusItem;
begin
	StdStatusKeys :=
		NewStatusKey ('', kbAltX, cmQuit,
		NewStatusKey ('', kbF10, cmMenu,
		NewStatusKey ('', kbAltF3, cmClose,
		NewStatusKey ('', kbF5, cmZoom,
		NewStatusKey ('', kbCtrlF5, cmResize,
		NewStatusKey ('', kbF6, cmNext,
		NewStatusKey ('', kbShiftF6, cmPrev, Next)))))));
end;

function StdFileMenuItems (Next : PMenuItem) : PMenuItem;
begin
	StdFileMenuItems :=
		NewItem ('&', '', kbNoKey, cmNew, hcNew,
		NewItem ('&...', 'F3', kbF3, cmOpen, hcOpen,
		NewItem ('&', 'F2', kbF2, cmSave, hcSave,
		NewItem ('& ...', '', kbNoKey, cmSaveAs, hcSaveAs,
		NewItem (' &', '', kbNoKey, cmSaveAll, hcSaveAll,
		NewLine (
		NewItem ('& ⠫...', '', kbNoKey, cmChangeDir, hcChangeDir,
		NewItem ('&맮 DOS', '', kbNoKey, cmDosShell, hcDosShell,
		NewItem ('&室', 'Alt+X', kbAltX, cmQuit, hcExit, Next)))))))));
end;

function StdEditMenuItems (Next : PMenuItem) : PMenuItem;
begin
	StdEditMenuItems :=
		NewItem ('&⪠', '', kbAltBack, cmUndo, hcUndo,
		NewLine (
		NewItem ('&', 'Shift+Del', kbShiftDel, cmCut, hcCut,
		NewItem ('&஢', 'Ctrl+Ins', kbCtrlIns, cmCopy, hcCopy,
		NewItem ('&', 'Shift+Ins', kbShiftIns, cmPaste, hcPaste,
		NewItem ('&⮦', 'Ctrl+Del', kbCtrlDel, cmClear, hcClear,	Next))))));
end;

function StdWindowMenuItems (Next : PMenuItem) : PMenuItem;
begin
	StdWindowMenuItems :=
		NewItem ('&९楩', '', kbNoKey, cmTile, hcTile,
		NewItem ('&᪠', '', kbNoKey, cmCascade, hcCascade,
		NewItem (' &', '', kbNoKey, cmCloseAll, hcCloseAll,
		NewLine (
		NewItem ('/&','Ctrl+F5', kbCtrlF5, cmResize, hcResize,
		NewItem ('&ᯠ', 'F5', kbF5, cmZoom, hcZoom,
		NewItem ('&饥', 'F6', kbF6, cmNext, hcNext,
		NewItem ('&न饥', 'Shift+F6', kbShiftF6, cmPrev, hcPrev,
		NewItem ('&', 'Alt+F3', kbAltF3, cmClose, hcClose, Next)))))))));
end;

{********************************************** App registration procedure **}
procedure RegisterApp;
begin
	RegisterType (RBackground);
	RegisterType (RDeskTop);
end;

function TProgram.InitMouseAnimator (AGraph : Boolean) : Pointer;
begin
	InitMouseAnimator := NIL;
end;

end.
