{Copyright by Leonid Youriev, 1993, os@imfiko.bishkek.su}
{$I VERSION}

unit DataServ;

interface

{$I VERSION}

type

	TDate =	record
		case Integer of
			0 : (
					Day		: ShortInt;
					Mon		: ShortInt;
					Year	: Word;
				);
			1 : (__ : Longint);
	end;

	TTab = Set of 0..30;

	TTabAt = record
		At	: TDate;
		The	: TTab;
	end;

	TTime = record
		case Integer of
			0 : (
					Sec		: ShortInt;
					Min		: ShortInt;
					Hour	: ShortInt;
				);
			1 : (__ : Longint);
	end;

	procedure FormatDate (D : TDate; Var S : String);
	procedure FormatTime (T : TTime; Var S : String);
	function DaysInMonth (Mon : Byte; Year : Word) : Word;
	function SubTime (T1 : TTime; T2 : Longint) : LongInt;
	function AddTime (T1 : TTime; T2 : Longint) : Longint;
	function SubDate (D1 : TDate; D2 : Longint) : Longint;
	function AddDate (D1 : TDate; D2 : Longint) : Longint;
	function DateToInt (D : TDate) : Longint;
	function TimeToInt (T : TTime) : Longint;
	function IntToDate (L : Longint) : Longint;
	function IntToTime (L : Longint) : Longint;
	function TextToTime (C : String; Var T : TTime) : Boolean;
	function TextToDate (S : String; Var D : TDate) : Boolean;
	function DayOfWeek (D : TDate) : Word;
	function MonthName (Month : Byte; As : Boolean) : String;
	function ValX (S : String; Var D : Word) : Boolean;
	procedure NTime (Var T : TTime);
	procedure NDate (Var D : TDate);
	function SystemTime : Longint;
	function SystemDate : Longint;
	procedure SetTime (T : TTime);
	procedure SetDate (D : TDate);

const

	MonthNames : array [0..11] of String [8] =
		(
		'',
		'ࠫ',
		'',
		'५',
		'',
		'',
		'',
		'',
		'',
		'',
		'',
		''
		);

	EMonthNames : array [0..11] of String [9] =
		(
		'January',
		'February',
		'Mart',
		'April',
		'May',
		'June',
		'July',
		'August',
		'September',
		'October',
		'Nowember',
		'December'
		);

	SEMonthNames : array [0..11] of String [3] =
		(
		'Jan',
		'Feb',
		'Mar',
		'Apr',
		'May',
		'Jun',
		'Jul',
		'Aug',
		'Sep',
		'Oct',
		'Now',
		'Dec'
		);

		EDayNames : array [0..6] of String [3] =
		(
		'Mon',
		'Tue',
		'Wed',
		'Thu',
		'Fri',
		'Sat',
		'Sun'
		);

   CDaysInMonth: array[0..11] of Byte =
	 (31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31);

implementation

uses	CharLog, Dos;

function MonthName (Month : Byte; As : Boolean) : String;
begin
	while Month > 11 do Dec (Month, 12);
	if AS then
		if (Month = 2) or (Month = 7) then
			MonthName := MonthNames [Month] + ''
		else
			MonthName := Copy (MonthNames [Month], 1, Length (MonthNames [Month]) - 1) + ''
	else
		MonthName := MonthNames [Month];
end;


function SystemTime : Longint;
Var
	H,
	M,
	S,
	c	: Word;
	T	: TTime;
begin
	Dos.GetTime (H, M, S, c);
	with T do
		begin
			Hour := H;
			Min := M;
			Sec := S;
		end;
	SystemTime := T.__;
end;

function SystemDate : Longint;
Var
	M,
	De,
	w	: Word;
	D	: TDate;
begin
	Dos.GetDate (D.Year, M, De, w);
	Dec (De); Dec (M);
	with D do
		begin
			Mon := M;
			Day := De;
		end;
	SystemDate := D.__;
end;

procedure SetTime (T : TTime);
begin
	NTime (T);
	Dos.SetTime (T.Hour, T.Min, T.Sec, 0);
end;

procedure SetDate (D : TDate);
begin
	NDate (D);
	Inc (D.Day);
	Inc (D.Mon);
	Dos.SetDate (D.Year, D.Mon, D.Day);
end;

procedure NTime (Var T : TTime);
begin
	T.__ := IntToTime (TimeToInt (T));
	T.Hour := T.Hour mod 24;
end;

procedure NDate (Var D : TDate);
begin
	D.__ := IntToDate (DateToInt (D));
end;

procedure FormatDate (D : TDate; Var S : String);
Var
	C	: String [7];
	K	: Integer;
begin
	NDate (D);
	Str (D.Day + 1 : 2, S);
	S := S + ' ' + MonthNames [D.Mon];
	k := Ord (S [0]);
	if (S [k] = '') or (S [k] = '') then S [k] := '' else S := S + '';
	Str (D.Year, C);
	S := S + ' ' + C;
end;

procedure FormatTime (T : TTime; Var S : String);
Var
	C	: String [7];
begin
	NTime (T);
	Str (T.Hour : 2, S);
	Str (T.Min : 2, C);
	S := S + ':' + C;
	Str (T.Sec : 2, C);
	S := S + ':' + C;
	ReplaceChar (S, ' ', 'O');
end;

function DaysInMonth (Mon : Byte; Year : Word) : Word;
Var
	D	: Byte;
begin
	D := CDaysInMonth [Mon];
	if (Mon = 1) and (Year and $0003 = 0) then Inc (D);
	DaysInMonth := D;
end;

function SubTime (T1 : TTime; T2 : Longint) : Longint;
Var
	T	: TTime absolute T2;
begin
	SubTime := Longint (IntToTime (TimeToInt (T1) - TimeToInt (T)));
end;

function AddTime (T1 : TTime; T2 : Longint) : Longint;
Var
	T	: TTime absolute T2;
begin
	AddTime := Longint (IntToTime (TimeToInt (T1) + TimeToInt (T)));
end;

function AddDate (D1 : TDate; D2 : Longint) : Longint;
Var
	D	: TDate absolute D2;
begin
	AddDate := Longint (IntToDate (DateToInt (D1) + DateToInt (D)));
end;

function SubDate (D1 : TDate; D2 : Longint) : Longint;
Var
	D	: TDate absolute D2;
begin
	SubDate := Longint (IntToDate (DateToInt (D1) - DateToInt (D)));
end;

function DateToInt (D : TDate) : Longint;
Var
	L	: Longint;
	I	: Integer;
begin
	L := Longint (D.Year) * 365 + D.Year shr 2;
	for I := 0 to D.Mon - 1 do
		Inc (L, CDaysInMonth [I]);
	DateToInt := L + D.Day;
end;

function IntToDate (L : Longint) : Longint;
Var
	I	: Integer;
	D 	: TDate;
Label
	St;
begin
	D.Year := L div 365; L := L mod 365;
	while L < D.Year shr 2 do
		begin
			Dec (D.Year);
			Inc (L, 365);
		end;
	L := L - D.Year shr 2;
	D.Mon := 0;
	St:
	I := DaysInMonth (D.Mon, D.Year);
	if L >= I then
		begin
			Inc (D.Mon);
			Dec (L, I);
			goto St;
		end;
	D.Day := L;
	IntToDate := D.__;
end;

function TimeToInt (T : TTime) : Longint;
begin
	TimeToInt := Longint(Integer(T.Hour) * 60 + Integer(T.Min)) * 60 + T.Sec;
end;

function IntToTime (L : Longint) : Longint;
Var
	T	: TTime;
begin
	T.Hour := Word (L div 3600);
	T.Min := Word ((L mod 3600) div 60);
	T.Sec := Word (L mod 60);
	IntToTime := T.__;
end;

function TextToTime (C : String; Var T : TTime) : Boolean;
Var
	I,
	N	: Integer;
	B	: Boolean;
	H,
	M,
	S	: Word;
begin
	TextToTime := False;
	for I := 1  to Length (C) do
		if not (C[I] in [' ', #9, ':', '0'..'9']) then Exit;
	I := Pos (':', C);
	if I < 2 then Exit;
	C [I] := ' ';
	N := Pos (':', C);
	if N > 0 then
		begin
			if N < 4 then Exit;
			C [N] := ' ';
			if Pos (':', C) > 0 then Exit;
		end;
	B := ValX (C, H);
	Delete (C, 1, I);
	B := ValX (C, M) and B;
	Delete (C, 1, N-I);
	TextToTime := ValX (C, S) and B;
	with T do
		begin
			Hour := H;
			Min := M;
			Sec := S;
		end;
	NTime (T);
end;

function TextToDate (S : String; Var D : TDate) : Boolean;
Var
	I	: Integer;
	B	: Boolean;
	M,
	De	: Word;

	procedure Cut;
	begin
		while (S[1] <= ' ') and (Ord(S[0]) > 0) do Delete (S, 1, 1);
	end;

begin
	TextToDate := False;
	Cut;
	ReplaceChar (S, '/', ' ');
	ReplaceChar (S, '\', ' ');
	I := Pos (' ', S);
	if I < 2 then Exit;
	S [I] := ' ';
	B := ValX (S, De); if B then Dec (De);
	Delete (S, 1, I);
	Cut;
	if not ValX (S, M) then
		begin
			M := 0;
			for I := 1 to 3 do
				if (Ord (S[0]) < I) or (S[I] <> MonthNames[M][I]) then
					begin
						I := 0;
						if M >= 11 then Exit;
						Inc (M);
					end;
		end
	else
		Dec (M);
	I := Pos ('/', S);
	if I < 1 then I := Pos (' ', S);
	if I < 2 then Exit;
	S [I] := ' ';
	Delete (S, 1, I);
	Cut;
	TextToDate := ValX (S, D.Year) and B;
	with D do
		begin
			Mon := M;
			Day := De;
		end;
	NDate (D);
end;

function DayOfWeek (D : TDate) : Word;
begin
	DayOfWeek := (DateToInt (D) + 5) mod 7;
end;

function ValX (S : String; Var D : Word) : Boolean;
Var
	I	: Integer;
begin
	I := 1;
	while (I <= Ord (S[0])) and ((S[I] >= '0') and (S[I] <= '9')) do
		Inc (I);
	if I <= Ord (S[0]) then S[0] := Char (I - 1);
	Val (S, D, I);
	ValX := I = 0;
end;

end.