{1/2 Copyright by Leonid Youriev 1992, 93}
{$I VERSION}

unit Dialogs;

interface

uses	Objects, Drivers, Views, CharLog, Quick, Validate, Sysvars, Styles;

{$I VERSION}

const

{ Color palettes }
	CStaticText		= #7;
	CLabel			= #7#8#9#9;
	CCluster		= #16#17#18#31;
	CInputLine		= #32#19#20#21;
	CHistory		= #22#23;
	CHistoryWindow	= #19#19#21#24#25#19#20;
	CHistoryViewer	= #6#6#7#6#6;

{ TDialog palette entires }
	dpGrayDialog	= 0;
	dpBlueDialog	= 1;
	dpCyanDialog	= 2;

{ TMultiCheckboxes flags }
{ hiword = number of bits }
{ loword = bit mask }
	cfOneBit		= $0101;
	cfTwoBits		= $0203;
	cfFourBits		= $040F;
	cfEightBits		= $08FF;

type

{ TDialog }
	{ Palette layout }
	{ 1 = Frame passive }
	{ 2 = Frame active }
	{ 3 = Frame icon }
	{ 4 = ScrollBar page area }
	{ 5 = ScrollBar controls }
	{ 6 = StaticText }
	{ 7 = Label normal }
	{ 8 = Label selected }
	{ 9 = Label shortcut }
	{ 10 = Button normal }
	{ 11 = Button default }
	{ 12 = Button selected }
	{ 13 = Button disabled }
	{ 14 = Button shortcut }
	{ 15 = Button shadow }
	{ 16 = Cluster normal }
	{ 17 = Cluster selected }
	{ 18 = Cluster shortcut }
	{ 19 = InputLine normal text }
	{ 20 = InputLine selected text }
	{ 21 = InputLine arrows }
	{ 22 = History arrow }
	{ 23 = History sides }
	{ 24 = HistoryWindow scrollbar page area }
	{ 25 = HistoryWindow scrollbar controls }
	{ 26 = ListViewer normal }
	{ 27 = ListViewer focused }
	{ 28 = ListViewer selected }
	{ 29 = ListViewer divider }
	{ 30 = InfoPane }
	{ 31 = Cluster disabled }
	{ 32 = InputLine focused text  }

	PDialog = ^TDialog;
	TDialog = Object (TWindow)
		constructor	Init (var Bounds : TRect; ATitle : TTitleStr);
		procedure	HandleEvent (var Event : TEvent); virtual;
		function	Valid (Command : Word) : Boolean; virtual;
		procedure	Labeled (AText : String; ALink : PView);
	end;

{ TSItem }
	PSItem = ^TSItem;
	TSItem = record
		Value	: PString;
		Next	: PSItem;
	end;

{ TInputLine }
	{ Palette layout }
	{ 1 = Passive }
	{ 2 = Active }
	{ 3 = Selected }
	{ 4 = Arrows }

	PInputLine = ^TInputLine;
	TInputLine = Object (TView)
		Data		: PString;
		MaxLen		: Integer;
		CurPos		: Integer;
		FirstPos	: Integer;
		SelStart	: Integer;
		SelEnd		: Integer;
		Validator	: PValidator;
		constructor	Init (var Bounds : TRect; AMaxLen : Integer);
		constructor	Load (var S : TStream);
		destructor	Done; virtual;
		function	DataSize : Word; virtual;
		procedure	Draw; virtual;
		procedure	GetData (var Rec); virtual;
		procedure	HandleEvent (var Event : TEvent); virtual;
		procedure	SelectAll (Enable : Boolean);
		procedure	SetData (var Rec); virtual;
		procedure	SetState (AState : Word; Enable : Boolean); virtual;
		procedure	SetValidator (AValid : PValidator);
		procedure	Store (var S : TStream);
		function	Valid (Command : Word) : Boolean; virtual;
	private
		function	CanScroll (Delta : Integer) : Boolean;
	end;

{ TCluster }
{ Palette layout }
	{ 1 = Normal text }
	{ 2 = Selected text }
	{ 3 = Normal shortcut }
	{ 4 = Selected shortcut }

	PCluster = ^TCluster;
	TCluster = Object (TView)
		Value		: Longint;
		Sel			: Integer;
		EnableMask	: Longint;
		Strings		: TStringCollection;
		constructor	Init (var Bounds : TRect; AStrings : PSItem);
		constructor	Load (var S : TStream);
		destructor	Done; virtual;
		function	ButtonState (Item : Integer) : Boolean;
		function	DataSize : Word; virtual;
{$IFNDEF Turbo6}
		procedure	DrawMultiBox (const Icon, Marker : String);
{$ELSE}
		procedure	DrawMultiBox (Icon, Marker : String);
{$ENDIF}
		procedure	DrawBox (Icon : String; Marker : Char);
		procedure	GetData (var Rec); virtual;
		function	GetHelpCtx : Word; virtual;
		procedure	HandleEvent (var Event : TEvent); virtual;
		function	Mark (Item : Integer) : Boolean; virtual;
		function	MultiMark (Item : Integer) : Byte; virtual;
		procedure	Press (Item : Integer); virtual;
		procedure	MovedTo (Item : Integer); virtual;
		procedure	SetButtonState (AMask : Longint; Enable : Boolean);
		procedure	SetData (var Rec); virtual;
		procedure	SetState (AState : Word; Enable : Boolean); virtual;
		procedure	Store (var S : TStream);
	private
		function	Column (Item : Integer) : Integer;
		function	FindSel (P : TPoint) : Integer;
		function	Row (Item : Integer) : Integer;
	end;

{ TRadioButtons }
{ Palette layout }
	{ 1 = Normal text }
	{ 2 = Selected text }
	{ 3 = Normal shortcut }
	{ 4 = Selected shortcut }

	PRadioButtons = ^TRadioButtons;
	TRadioButtons = Object (TCluster)
		procedure	Draw; virtual;
		function	Mark (Item : Integer) : Boolean; virtual;
		procedure	MovedTo (Item : Integer); virtual;
		procedure	Press (Item : Integer); virtual;
		procedure	SetData (var Rec); virtual;
	end;

{ TCheckBoxes }
{ Palette layout }
	{ 1 = Normal text }
	{ 2 = Selected text }
	{ 3 = Normal shortcut }
	{ 4 = Selected shortcut }

	PCheckBoxes = ^TCheckBoxes;
	TCheckBoxes = Object (TCluster)
		procedure	Draw; virtual;
		function	Mark (Item : Integer) : Boolean; virtual;
		procedure	Press (Item : Integer); virtual;
	end;

{ TMultiCheckBoxes }
	{ Palette layout }
	{ 1 = Normal text }
	{ 2 = Selected text }
	{ 3 = Normal shortcut }
	{ 4 = Selected shortcut }

	PMultiCheckBoxes = ^TMultiCheckBoxes;
	TMultiCheckBoxes = Object (TCluster)
		SelRange	: Byte;
		Flags		: Word;
		States		: PString;
{$IFNDEF Turbo6}
		constructor	Init (var Bounds : TRect; AStrings : PSItem; ASelRange : Byte; AFlags : Word; const AStates : String);
{$ELSE}
		constructor	Init (var Bounds : TRect; AStrings : PSItem; ASelRange : Byte; AFlags : Word; AStates : String);
{$ENDIF}
		constructor	Load (var S : TStream);
		destructor	Done; virtual;
		function	DataSize : Word; virtual;
		procedure	Draw; virtual;
		procedure	GetData (var Rec); virtual;
		function	MultiMark (Item : Integer) : Byte; virtual;
		procedure	Press (Item : Integer); virtual;
		procedure	SetData (var Rec); virtual;
		procedure	Store (var S : TStream);
	end;

{ TListBox }
	{ Palette layout }
	{ 1 = Active }
	{ 2 = Inactive }
	{ 3 = Focused }
	{ 4 = Selected }
	{ 5 = Divider }

	PListBox = ^TListBox;
	TListBox = Object (TListViewer)
		List		: PCollection;
		constructor	Init (var Bounds : TRect; ANumCols : Word; AScrollBar : PScrollBar);
		constructor	Load (var S : TStream);
		function	DataSize : Word; virtual;
		procedure	GetData (var Rec); virtual;
		function	GetText (Item : Integer; MaxLen : Integer) : String; virtual;
		procedure	NewList (AList : PCollection); virtual;
		procedure	SetData (var Rec); virtual;
		procedure	Store (var S : TStream);
	end;

{ TStaticText }
{ Palette layout }
	{ 1 = Text }
	{ 2 = ~Text }

	PStaticText = ^TStaticText;
	TStaticText = Object (TView)
		Text		: PString;
		constructor	Init (var Bounds : TRect; AText : String);
		constructor	Load (var S : TStream);
		destructor	Done; virtual;
		procedure	Draw; virtual;
		procedure	GetText (var S : String); virtual;
		procedure	Store (var S : TStream);
	end;

{ TParamText }
{ Palette layout }
	{ 1 = Text }
	{ 2 = ~Text }

	PParamText = ^TParamText;
	TParamText = Object (TStaticText)
		ParamCount	: Integer;
		ParamList	: Pointer;
		constructor	Init (var Bounds : TRect; AText : String; AParamCount : Integer);
		constructor	Load (var S : TStream);
		function	DataSize : Word; virtual;
		procedure	GetText (var S : String); virtual;
		procedure	SetData (var Rec); virtual;
		procedure	Store (var S : TStream);
	end;

{ TLabel }
{ Palette layout }
	{ 1 = Normal text }
	{ 2 = Selected text }
	{ 3 = Normal shortcut }
	{ 4 = Selected shortcut }

	PLabel = ^TLabel;
	TLabel = Object (TStaticText)
		Link		: PView;
		Light		: Boolean;
		constructor	Init (var Bounds : TRect; AText : String; ALink : PView);
		constructor	Load (var S : TStream);
		procedure	Draw; virtual;
		procedure	HandleEvent (var Event : TEvent); virtual;
		procedure	Store (var S : TStream);
	end;

{ THistoryViewer }
{ Palette layout }
	{ 1 = Active }
	{ 2 = Inactive }
	{ 3 = Focused }
	{ 4 = Selected }
	{ 5 = Divider }

	PHistoryViewer = ^THistoryViewer;
	THistoryViewer = Object (TListViewer)
		HistoryId	: Word;
		constructor Init (var Bounds : TRect; AHScrollBar, AVScrollBar : PScrollBar; AHistoryId : Word);
		function	GetText (Item : Integer; MaxLen : Integer) : String; virtual;
		procedure	HandleEvent (var Event : TEvent); virtual;
		function	HistoryWidth : Integer; virtual;
	end;

{ THistoryWindow }
{ Palette layout }
	{ 1 = Frame passive }
	{ 2 = Frame active }
	{ 3 = Frame icon }
	{ 4 = ScrollBar page area }
	{ 5 = ScrollBar controls }
	{ 6 = HistoryViewer normal text }
	{ 7 = HistoryViewer selected text }

	PHistoryWindow = ^THistoryWindow;
	THistoryWindow = Object (TWindow)
		Viewer		: PListViewer;
		constructor	Init (var Bounds : TRect; HistoryId : Word);
		function	GetSelection : String; virtual;
		procedure	InitViewer (HistoryId : Word); virtual;
	end;

{ THistory }
{ Palette layout }
	{ 1 = Arrow }
	{ 2 = Sides }

	PHistory = ^THistory;
	THistory = Object (TView)
		Link		: PInputLine;
		HistoryId	: Word;
		constructor	Init (var Bounds : TRect; ALink : PInputLine; AHistoryId : Word);
		constructor	Load (var S : TStream);
		procedure	Draw; virtual;
		procedure	HandleEvent (var Event : TEvent); virtual;
		function	InitHistoryWindow (var Bounds : TRect) : PHistoryWindow; virtual;
		procedure	Store (var S : TStream);
	end;

{ SItem routines }
function NewSItem (Str : String; ANext : PSItem) : PSItem;

{ Dialogs registration procedure }
procedure RegisterDialogs;

{********************************************* Stream Registration Records **}
const

	RDialog		: TStreamRec = (
		ObjType	: 10;
		VmtLink	: Ofs (TypeOf (TDialog)^);
		Load	: @TDialog.Load;
		Store	: @TDialog.Store);

	RInputLine	: TStreamRec = (
		ObjType	: 11;
		VmtLink	: Ofs (TypeOf (TInputLine)^);
		Load	: @TInputLine.Load;
		Store	: @TInputLine.Store);

	RButton		: TStreamRec = (
		ObjType	: 12;
		VmtLink : Ofs (TypeOf (TButton)^);
		Load	: @TButton.Load;
		Store	: @TButton.Store);

	RCluster	: TStreamRec = (
		ObjType	: 13;
		VmtLink	: Ofs (TypeOf (TCluster)^);
		Load	: @TCluster.Load;
		Store	: @TCluster.Store);

	RRadioButtons	: TStreamRec = (
		ObjType	: 14;
		VmtLink	: Ofs (TypeOf (TRadioButtons)^);
		Load	: @TRadioButtons.Load;
		Store	: @TRadioButtons.Store);

	RCheckBoxes	: TStreamRec = (
		ObjType	: 15;
		VmtLink	: Ofs (TypeOf (TCheckBoxes)^);
		Load	: @TCheckBoxes.Load;
		Store	: @TCheckBoxes.Store);

	RListBox	: TStreamRec = (
		ObjType	: 16;
		VmtLink	: Ofs (TypeOf (TListBox)^);
		Load	: @TListBox.Load;
		Store	: @TListBox.Store);

	RStaticText	: TStreamRec = (
		ObjType	: 17;
		VmtLink	: Ofs (TypeOf (TStaticText)^);
		Load	: @TStaticText.Load;
		Store	: @TStaticText.Store);

	RLabel		: TStreamRec = (
		ObjType	: 18;
		VmtLink	: Ofs (TypeOf (TLabel)^);
		Load	: @TLabel.Load;
		Store	: @TLabel.Store);

	RHistory	: TStreamRec = (
		ObjType	: 19;
		VmtLink	: Ofs (TypeOf (THistory)^);
		Load	: @THistory.Load;
		Store	: @THistory.Store);

	RParamText	: TStreamRec = (
		ObjType	: 20;
		VmtLink	: Ofs (TypeOf (TParamText)^);
		Load	: @TParamText.Load;
		Store	: @TParamText.Store);

	RMultiCheckBoxes : TStreamRec = (
		ObjType	: 27;
		VmtLink	: Ofs (TypeOf (TMultiCheckBoxes)^);
		Load	: @TMultiCheckBoxes.Load;
		Store	: @TMultiCheckBoxes.Store);

implementation

uses	HistList, App;

{***************************************************************** TDialog **}
constructor TDialog.Init (var Bounds : TRect; ATitle : TTitleStr);
begin
{$IFNDEF Turbo6}
	inherited Init (Bounds, ATitle, wnNoNumber);
{$ELSE}
	TWindow.Init (Bounds, ATitle, wnNoNumber);
{$ENDIF}
	GrowMode := 0;
{	Palette := 0;
}	Flags := wfMove or wfClose or wfWinTitle;
	InsertSoftGrid;
end;

procedure TDialog.HandleEvent (var Event : TEvent);
begin
{$IFNDEF Turbo6}
	inherited HandleEvent (Event);
{$ELSE}
	TWindow.HandleEvent (Event);
{$ENDIF}
	case Event.What of
		evKeyDown	:
			case Event.KeyCode of
				kbEsc	: SendCommand (cmCancel);
				kbEnter	: SendBroadcast (cmDefault);
			else
				Exit;
			end;
		evCommand	:
			case Event.Command of
				cmOk,
				cmCancel,
				cmYes,
				cmNo	: if State and sfModal <> 0 then EndModal (Event.Command);
			else
				Exit;
			end;
		else
			Exit;
		end;
	ClearEvent (Event);
end;

function TDialog.Valid (Command : Word) : Boolean;
begin
	if Command = cmCancel then
		Valid := True
	else
{$IFNDEF Turbo6}
		Valid := inherited Valid (Command);
{$ELSE}
		Valid := TWindow.Valid (Command);
{$ENDIF}
end;

procedure TDialog.Labeled (AText : String; ALink : PView);
var
	L	: PLabel;
	R	: TRect;
	I	: Integer;
begin
	if PtrRec (ALink).Seg = 0 then Exit;
	with ALink^ do
		begin
			R.A.X := Origin.X;
			R.A.Y := Origin.Y - 1;
			R.B.X := R.A.X + Size.X;
			R.B.Y := Origin.Y;
		end;
	repeat
		I := CharPos (AText, '^');
		if I > 0 then
			begin
				Inc (R.B.X, 3);
				AText [I] := ' ';
			end;
	until I <= 0;
	New (L, Init (R, AText, ALink));
	Insert (ALink);
	Insert (L);
end;

function NewSItem (Str : String; ANext : PSItem) : PSItem;
var
	Item	: PSItem;
begin
	New (Item);
	Item^.Value := NewStr (Str);
	Item^.Next := ANext;
	NewSItem := Item;
end;

{************************************************************** TInputLine **}
constructor TInputLine.Init (var Bounds : TRect; AMaxLen : Integer);
begin
{$IFNDEF Turbo6}
	inherited Init (Bounds);
{$ELSE}
	TView.Init (Bounds);
{$ENDIF}
	State := State or sfCursorVis;
	Options := Options or (ofSelectable + ofFirstClick + ofEditable);
	GetMem (Data, AMaxLen + 1);
	Data^ := '';
	MaxLen := AMaxLen;
end;

constructor TInputLine.Load (var S : TStream);
begin
{$IFNDEF Turbo6}
	inherited Load (S);
{$ELSE}
	TView.Load (S);
{$ENDIF}
	S.Read (MaxLen, SizeOf (Integer) * 5);
	GetMem (Data, MaxLen + 1);
	S.Read (Data^ [0], 1);
	S.Read (Data^ [1], Length (Data^));
	Validator := PValidator (S.Get);
end;

destructor TInputLine.Done;
begin
	Validator^.Free;
	FreeMem (Data, MaxLen + 1);
{$IFNDEF Turbo6}
	inherited Done;
{$ELSE}
	TView.Done;
{$ENDIF}
end;

function TInputLine.CanScroll (Delta : Integer) : Boolean;
begin
	if Delta < 0 then
		CanScroll := FirstPos > 0
	else
		if Delta > 0 then
			CanScroll := Length (Data^) - FirstPos + 2 > Size.X
		else
			CanScroll := False;
end;

function TInputLine.DataSize : Word;
var
	DSize	: Word;
begin
	DSize := 0;
	if PtrRec (Validator).Seg <> 0 then
		DSize := Validator^.Transfer (Data^, NIL, vtDataSize);
	if DSize <> 0 then
		DataSize := DSize
	else
		DataSize := MaxLen + 1;
end;

procedure TInputLine.Draw;
var
	Color	: Byte;
	L,
	R		: Integer;
	B		: TDrawBuffer;
begin
	if State and sfFocused = 0 then
		Color := Lo (GetColor ($12))
	else
		Color := Lo (GetColor ($1F));
	MoveChar (B, ' ', Color, Size.X);
	MoveStr (B [1], Copy (Data^, FirstPos + 1, Size.X - 2), Color);
	if CanScroll (1) then MoveChar (B [Size.X - 1], #16, Lo (GetColor ($14)), 1);
	if State and sfSelected <> 0 then
		begin
			if CanScroll (-1) then MoveChar (B [0], #17, Lo (GetColor ($14)), 1);
			L := SelStart - FirstPos;
			R := SelEnd - FirstPos;
			if L < 0 then L := 0;
			if R > Size.X - 2 then R := Size.X - 2;
			if L < R then MoveChar (B [L + 1], #0, Lo (GetColor ($13)), R - L);
		end;
		WriteLine (0, 0, Size.X, Size.Y, B);
		SetCursor (CurPos - FirstPos + 1, 0);
end;

procedure TInputLine.GetData (var Rec);
begin
	if (PtrRec (Validator).Seg = 0) or (Validator^.Transfer (Data^, @Rec, vtGetData) = 0) then
		begin
			FillChar (Rec, DataSize, #0);
			Move (Data^, Rec, Length (Data^) + 1);
		end;
end;

procedure TInputLine.HandleEvent (var Event : TEvent);
const
	PadKeys		= [$47, $4B, $4D, $4F, $73, $74];
var
	Delta,
	Anchor,
	I			: Integer;
	ExtendBlock	: Boolean;
	OldData		: String;
	OldCurPos,
	OldFirstPos,
	OldSelStart,
	OldSelEnd	: Integer;
	WasAppending : Boolean;

	function MouseDelta : Integer;
	var
		Mouse	: TPoint;
	begin
		MakeLocal (Event.Where, Mouse);
		if Mouse.X <= 0 then
			MouseDelta := -1
		else
			if Mouse.X >= Size.X - 1 then
				MouseDelta := 1
			else
				MouseDelta := 0;
	end;

	function MousePos : Integer;
	var
		Pos		: Integer;
		Mouse	: TPoint;
	begin
		MakeLocal (Event.Where, Mouse);
		if Mouse.X < 1 then Mouse.X := 1;
		Pos := Mouse.X + FirstPos - 1;
		if Pos < 0 then Pos := 0;
		if Pos > Length (Data^) then Pos := Length (Data^);
		MousePos := Pos;
	end;

	procedure DeleteSelect;
	begin
		if SelStart <> SelEnd then
			begin
				Delete (Data^, SelStart + 1, SelEnd - SelStart);
				CurPos := SelStart;
			end;
	end;

	procedure AdjustSelectBlock;
	begin
		if CurPos < Anchor then
			begin
				SelStart := CurPos;
				SelEnd := Anchor;
			end
		else
			begin
				SelStart := Anchor;
				SelEnd := CurPos;
			end;
	end;

	procedure SaveState;
	begin
		if PtrRec (Validator).Seg <> 0 then
			begin
				OldData := Data^;
				OldCurPos := CurPos;
				OldFirstPos := FirstPos;
				OldSelStart := SelStart;
				OldSelEnd := SelEnd;
				WasAppending := Length (Data^) = CurPos;
			end;
	end;

	procedure RestoreState;
	begin
		if PtrRec (Validator).Seg <> 0 then
			begin
				Data^ := OldData;
				CurPos := OldCurPos;
				FirstPos := OldFirstPos;
				SelStart := OldSelStart;
				SelEnd := OldSelEnd;
			end;
	end;

	function CheckValid (NoAutoFill : Boolean) : Boolean;
	var
		OldLen	: Integer;
		NewData	: String;
	begin
		if PtrRec (Validator).Seg <> 0 then
			begin
				CheckValid := False;
				OldLen := Length (Data^);
				if (Validator^.Options and voOnAppend = 0) or (WasAppending and (CurPos = OldLen)) then
					begin
						NewData := Data^;
						if not Validator^.IsValidInput (NewData, NoAutoFill) then
							RestoreState
						else
							begin
								if Length (NewData) > MaxLen then NewData [0] := Char (MaxLen);
								Data^ := NewData;
								if (CurPos >= OldLen) and (Length (Data^) > OldLen) then
									CurPos := Length (Data^);
								CheckValid := True;
							end;
					end
				else
					begin
						CheckValid := True;
						if CurPos = OldLen then
						if not Validator^.IsValidInput (Data^, False) then
							begin
								Validator^.Error;
								CheckValid := False;
							end;
					end;
			end
		else
			CheckValid := True;
	end;

begin
{$IFNDEF Turbo6}
	inherited HandleEvent (Event);
{$ELSE}
	TView.HandleEvent (Event);
{$ENDIF}
	if State and sfSelected <> 0 then
		begin
			case Event.What of
				evMouseDown	:
					begin
						Delta := MouseDelta;
						if CanScroll (Delta) then
							begin
								repeat
									if CanScroll (Delta) then
										begin
											Inc (FirstPos, Delta);
											DrawView;
										end;
								until not MouseEvent (Event, evMouseAuto);
							end
						else
							if Event.Double then
								SelectAll (True)
							else
								begin
									Anchor := MousePos;
									repeat
										if Event.What = evMouseAuto then
											begin
												Delta := MouseDelta;
												if CanScroll (Delta) then Inc (FirstPos, Delta);
											end;
										CurPos := MousePos;
										AdjustSelectBlock;
										DrawView;
									until not MouseEvent (Event, evMouseMove + evMouseAuto);
								end;
					end;
				evKeyDown :
					begin
						SaveState;
						Event.KeyCode := CtrlToArrow (Event.KeyCode);
						if (Event.ScanCode in PadKeys) and (ShiftState and $03 <> 0) then
							begin
								Event.CharCode := #0;
								if CurPos = SelEnd then
									Anchor := SelStart
								else
									Anchor := SelEnd;
								ExtendBlock := True;
							end
						else
							ExtendBlock := False;
						case Event.KeyCode of
							kbLeft :
								if CurPos > 0 then Dec (CurPos);
							kbRight :
								if CurPos < Length (Data^) then
									begin
										Inc (CurPos);
										CheckValid (True);
									end;
							kbHome :
								CurPos := 0;
							kbEnd :
								begin
									CurPos := Length (Data^);
									CheckValid (True);
								end;
							kbBack :
								if (CurPos > 0) and (Options and ofEditable <> 0) then
									begin
										Delete (Data^, CurPos, 1);
										Dec (CurPos);
										if FirstPos > 0 then Dec (FirstPos);
										CheckValid (True);
									end;
							kbDel :
								if Options and ofEditable <> 0 then
									begin
										if SelStart = SelEnd then
											if CurPos < Length (Data^) then
												begin
													SelStart := CurPos;
													SelEnd := CurPos + 1;
												end;
										DeleteSelect;
										CheckValid (True);
									end;
							kbIns :
								SetState (sfCursorIns, State and sfCursorIns = 0);
						else
							if Options and ofEditable <> 0 then
								case Event.CharCode of
									' '..#255 :
										begin
											if State and sfCursorIns <> 0 then
												Delete (Data^, CurPos + 1, 1)
											else
												DeleteSelect;
											if CheckValid (True) then
												begin
													if Length (Data^) < MaxLen then
														begin
															if FirstPos > CurPos then FirstPos := CurPos;
															Inc (CurPos);
															Insert (Event.CharCode, Data^, CurPos);
														end;
													CheckValid (False);
												end;
										end;
									^Y :
										begin
											Data^ := '';
											CurPos := 0;
										end;
								else
									Exit;
								end
						end;
						if ExtendBlock then
							AdjustSelectBlock
						else
							begin
								SelStart := CurPos;
								SelEnd := CurPos;
							end;
						if FirstPos > CurPos then FirstPos := CurPos;
						I := CurPos - Size.X + 2;
						if FirstPos < I then FirstPos := I;
						DrawView;
					end;
			else
				Exit;
			end;
			ClearEvent (Event);
		end;
end;

procedure TInputLine.SelectAll (Enable : Boolean);
begin
	CurPos := 0;
	FirstPos := 0;
	SelStart := 0;
	if Enable then
		SelEnd := Length (Data^)
	else
		SelEnd := 0;
	DrawView;
end;

procedure TInputLine.SetData (var Rec);
begin
	if (PtrRec (Validator).Seg = 0) or (Validator^.Transfer (Data^, @Rec, vtSetData) = 0) then
		Move (Rec, Data^ [0], MaxLen + 1);
	SelectAll (True);
end;

procedure TInputLine.SetState (AState : Word; Enable : Boolean);
begin
{$IFNDEF Turbo6}
	inherited SetState (AState, Enable);
{$ELSE}
	TView.SetState (AState, Enable);
{$ENDIF}
	if (AState = sfSelected) or ((AState = sfActive) and (State and sfSelected <> 0)) then SelectAll (Enable);
end;

procedure TInputLine.SetValidator (AValid : PValidator);
begin
	Validator^.Free;
	Validator := AValid;
end;

procedure TInputLine.Store (var S : TStream);
begin
{$IFNDEF Turbo6}
	inherited Store (S);
{$ELSE}
	TView.Store (S);
{$ENDIF}
	S.Write (MaxLen, SizeOf (Integer) * 5);
	S.WriteStr (Data);
	S.Put (Validator);
end;

function TInputLine.Valid (Command : Word) : Boolean;
begin
{$IFNDEF Turbo6}
	Valid := inherited Valid (Command);
{$ELSE}
	Valid := TView.Valid (Command);
{$ENDIF}
	if (PtrRec (Validator).Seg <> 0) and (State and sfDisabled = 0) then
		if Command = cmValid then
			Valid := Validator^.Status = vsOk
		else
			if Command <> cmCancel then
				if not Validator^.Valid (Data^) then
					begin
						Select;
						Valid := False;
					end;
end;

{**************************************************************** TCluster **}
constructor TCluster.Init (var Bounds : TRect; AStrings : PSItem);
var
	I	: Integer;
	P	: PSItem;
begin
{$IFNDEF Turbo6}
	inherited Init (Bounds);
{$ELSE}
	TView.Init (Bounds);
{$ENDIF}
	Options := Options or (ofSelectable + ofFirstClick + ofPreProcess + ofPostProcess + ofEditable);
	I := 0;
	P := AStrings;
	while PtrRec (P).Seg <> 0 do
		begin
			Inc (I);
			P := P^.Next;
		end;
	Strings.Init (I, 0);
	while PtrRec (AStrings).Seg <> 0 do
		begin
			P := AStrings;
			Strings.AtInsert (Strings.Count, AStrings^.Value);
			AStrings := AStrings^.Next;
			Dispose (P);
		end;
	Value := 0;
	Sel := 0;
	SetCursor (2, 0);
	ShowCursor;
	EnableMask := $FFFFFFFF;
end;

constructor TCluster.Load (var S : TStream);
begin
{$IFNDEF Turbo6}
	inherited Load (S);
{$ELSE}
	TView.Load (S);
{$ENDIF}
	S.Read (Value, SizeOf (Longint) * 2 + SizeOf (Integer));
	Strings.Load (S);
	SetButtonState (0, True);
end;

destructor TCluster.Done;
var
	I	: Integer;
begin
	Strings.Done;
{$IFNDEF Turbo6}
	inherited Done;
{$ELSE}
	TView.Done;
{$ENDIF}
end;

function TCluster.ButtonState (Item : Integer) : Boolean; assembler;
asm
	XOR		AL, AL
	MOV		CX, Item
	CMP		CX, 31
	JA		@3
	MOV		AX, 1
	XOR		DX, DX
	JCXZ	@2
@1:	SHL		AX, 1
	RCL		DX, 1
	LOOP	@1
@2:	LES		DI, Self
	AND		AX, ES:[DI].TCluster.EnableMask.Word [0]
	AND		DX, ES:[DI].TCluster.EnableMask.Word [2]
	OR		AX, DX
	JZ		@3
	MOV		AL, 1
@3 :
end;

function TCluster.DataSize : Word;
begin
	DataSize := SizeOf (Word);
end;

procedure TCluster.DrawBox (Icon : String; Marker : Char);
begin
	DrawMultiBox (Icon, Icon [2] + Marker);
end;

{$IFNDEF Turbo6}
procedure TCluster.DrawMultiBox (const Icon, Marker : String);
{$ELSE}
procedure TCluster.DrawMultiBox (Icon, Marker : String);
{$ENDIF}
var
	I, J,
	Cur,
	Col		: Integer;
	CNorm,
	CSel,
	CDis,
	Color	: Word;
	B		: TDrawBuffer;
begin
	CNorm := GetColor ($110F);
	CSel := GetColor ($1110);
	CDis := GetColor ($1E1E);
	for I := 0 to Size.Y do
		begin
			MoveChar (B, ' ', Byte (CNorm), Size.X);
			for J := 0 to (Strings.Count - 1) div Size.Y + 1 do
				begin
					Cur := J * Size.Y + I;
					if Cur < Strings.Count then
						begin
							Col := Column (Cur);
							if (Col + CStrLen (PString (Strings.At (Cur))^) + 5 < Sizeof (TDrawBuffer) div SizeOf (Word)) and (Col < Size.X) then
								begin
									if not ButtonState (Cur) then
										Color := CDis
									else
										if (Cur = Sel) and (State and sfFocused <> 0) then
											Color := CSel
										else
											Color := CNorm;
									MoveChar (B [Col], ' ', Lo (Color), Size.X - Col);
									MoveStr (B [Col], Icon, Lo (Color));
									WordRec (B [Col + 1]).Lo := Byte (Marker [MultiMark (Cur) + 1]);
									MoveCStr (B [Col + Length (Icon)], PString (Strings.At (Cur))^, Color);
								end;
						end;
				end;
			WriteBuf (0, I, Size.X, 1, B);
		end;
	SetState (sfCursorIns, Config.ShowMarkers);
	SetCursor (Column (Sel) + 1, Row (Sel));
end;

procedure TCluster.GetData (var Rec);
begin
	Word (Rec) := Value;
	DrawView;
end;

function TCluster.GetHelpCtx : Word;
begin
	GetHelpCtx := HelpCtx + Sel;
end;

procedure TCluster.HandleEvent (var Event : TEvent);
var
	Mouse	: TPoint;
	I, S	: Integer;
	C		: Char;
label
	Lx;

	procedure MoveSel;
	begin
		if I <= Strings.Count then
			begin
				Sel := S;
				MovedTo (Sel);
				DrawView;
			end;
	end;

begin
{$IFNDEF Turbo6}
	inherited HandleEvent (Event);
{$ELSE}
	TView.HandleEvent (Event);
{$ENDIF}
	if Event.What = evMouseDown then
		begin
			MakeLocal (Event.Where, Mouse);
			I := FindSel (Mouse);
			if (I > -1) and ButtonState (I) then
				Sel := I;
			DrawView;
			repeat
				MakeLocal (Event.Where, Mouse);
				if FindSel (Mouse) = Sel then
					ShowCursor
				else
					HideCursor;
			until not MouseEvent (Event, evMouseMove);
			ShowCursor;
			MakeLocal (Event.Where, Mouse);
			if (FindSel (Mouse) = Sel) and ButtonState (Sel) then
				begin
					Press (Sel);
					DrawView;
				end;
		end
	else
		if Event.What = evKeyDown then
			begin
				S := Sel;
				case Event.KeyCode of
					kbUp	:
						if (State and sfFocused <> 0) and (Size.Y > 1) then
							begin
								I := 0;
								repeat
									Inc (I);
									Dec (S);
									if S < 0 then S := Strings.Count - 1;
								until ButtonState (S) or (I > Strings.Count);
								MoveSel;
							end
						else
							Exit;
					kbDown	:
						if (State and sfFocused <> 0) and (Size.Y > 1) then
							begin
								I := 0;
								repeat
									Inc (I);
									Inc (S);
									if S >= Strings.Count then S := 0;
								until ButtonState (S) or (I > Strings.Count);
								MoveSel;
							end
						else
							Exit;
					kbRight	:
						if (State and sfFocused <> 0) and (Strings.Count > Size.Y) then
							begin
								I := 0;
								repeat
									Inc (I);
									Inc (S, Size.Y);
									if S >= Strings.Count then
										begin
											S := (S + 1) mod Size.Y;
											if S >= Strings.Count then S := 0;
										end;
								until ButtonState (S) or (I > Strings.Count);
								MoveSel;
							end
						else
							Exit;
					kbLeft	:
						if (State and sfFocused <> 0) and (Strings.Count > Size.Y) then
							begin
								I := 0;
								repeat
									Inc (I);
									if S > 0 then
										begin
											Dec (S, Size.Y);
											if S < 0 then
												begin
													S := ((Strings.Count + Size.Y - 1) div Size.Y)*Size.Y + S - 1;
													if S >= Strings.Count then
														S := Strings.Count-1;
												end;
										end
									else
										S := Strings.Count - 1;
								until ButtonState (S) or (I > Strings.Count);
								MoveSel;
							end
						else
							Exit;
				else
					for I := 0 to Strings.Count - 1 do
						begin
							C := HotKey (PString (Strings.At (I))^);
							if (GetAltCode (C) = Event.KeyCode) or (( (Owner^.Phase = phPostProcess)
							or (State and sfFocused <> 0)) and (C <> #0) and (KeyCase (Event.CharCode) = C)) then
								if ButtonState (I) then
									begin
										if Focus then
											begin
												Sel := I;
												MovedTo (I);
												Press (I);
											end;
										goto Lx;
									end
								else
									Exit;
						end;
					if (Event.CharCode = ' ') and (State and sfFocused <> 0) then
						begin
							Press (Sel);
							Lx :
							DrawView;
							ClearEvent (Event);
						end;
					Exit;
				end;
				MoveSel;
			end
		else
			Exit;
	ClearEvent (Event);
end;

procedure TCluster.SetButtonState (AMask : Longint; Enable : Boolean); assembler;
asm
	LES		DI, Self
	MOV		AX, AMask.Word [0]
	MOV		DX, AMask.Word [2]
	TEST	Enable, 0FFH
	JNZ		@1
	NOT		AX
	NOT		DX
	AND		ES:[DI].TCluster.EnableMask.Word [0], AX
	AND		ES:[DI].TCluster.EnableMask.Word [2], DX
	JMP		@2
@1:	OR		ES:[DI].TCluster.EnableMask.Word [0], AX
	OR		ES:[DI].TCluster.EnableMask.Word [2], DX
@2:	MOV		CX, ES:[DI].Strings.TCollection.Count
	CMP		CX, 32
	JA		@6
	MOV		BX, ES:[DI].TCluster.Options
	AND		BX, not ofSelectable
	MOV		AX, ES:[DI].TCluster.EnableMask.Word [0]
	MOV		DX, ES:[DI].TCluster.EnableMask.Word [2]
@3:	SHR		DX, 1
	RCR		AX, 1
	JC		@4
	LOOP	@3
	JMP		@5
@4:	OR		BX, ofSelectable
@5:	MOV		ES:[DI].TCluster.Options, BX
@6:
end;

procedure TCluster.SetData (var Rec);
begin
	Value := Word (Rec);
	DrawView;
end;

procedure TCluster.SetState (AState : Word; Enable : Boolean);
begin
	TView.SetState (AState, Enable);
	if AState = sfSelected then DrawView;
end;

function TCluster.Mark (Item : Integer) : Boolean;
begin
	Mark := False;
end;

function TCluster.MultiMark (Item : Integer) : Byte;
begin
	MultiMark := Byte (Mark (Item) = True);
end;

procedure TCluster.MovedTo (Item : Integer);
begin
end;

procedure TCluster.Press (Item : Integer);
begin
end;

procedure TCluster.Store (var S : TStream);
begin
{$IFNDEF Turbo6}
	inherited Store (S);
{$ELSE}
	TView.Store (S);
{$ENDIF}
	S.Write (Value, SizeOf (Longint) * 2 + SizeOf (Integer));
	Strings.Store (S);
end;

function TCluster.Column (Item : Integer) : Integer;
var
	I,
	Col,
	Width,
	L		: Integer;
begin
	if Item < Size.Y then
		Column := 0
	else
		begin
			Width := 0;
			Col := -6;
			for I := 0 to Item do
				begin
					if I mod Size.Y = 0 then
						begin
							Inc (Col, Width + 6);
							Width := 0;
						end;
					if I < Strings.Count then L := Length (PString (Strings.At (I))^);
					if L > Width then Width := L;
				end;
			Column := Col;
		end;
end;

function TCluster.FindSel (P : TPoint) : Integer;
var
	I,
	S	: Integer;
	R	: TRect;
begin
	GetExtent (R);
	if not R.Contains (P) then
		FindSel := -1
	else
		begin
			I := 0;
			while P.X >= Column (I + Size.Y) do
				Inc (I, Size.Y);
			S := I + P.Y;
			if S >= Strings.Count then
				FindSel := -1
			else
				FindSel := S;
		end;
end;

function TCluster.Row (Item : Integer) : Integer;
begin
	Row := Item mod Size.Y;
end;

{*********************************************************** TRadioButtons **}
procedure TRadioButtons.Draw;
begin
	DrawBox (DRIVERS._RaI, DRIVERS._RaB);
end;

function TRadioButtons.Mark (Item : Integer) : Boolean;
begin
	Mark := Item = Value;
end;

procedure TRadioButtons.Press (Item : Integer);
begin
	if Options and ofEditable <> 0 then Value := Item;
end;

procedure TRadioButtons.MovedTo (Item : Integer);
begin
	Value := Item;
end;

procedure TRadioButtons.SetData (var Rec);
begin
{$IFNDEF Turbo6}
	inherited SetData (Rec);
{$ELSE}
	TCluster.SetData (Rec);
{$ENDIF}
	Sel := Value;
end;

{************************************************************* TCheckBoxes **}

procedure TCheckBoxes.Draw;
begin
	DrawBox (DRIVERS._ChI, DRIVERS._ChB);
end;

function TCheckBoxes.Mark (Item : Integer) : Boolean;
begin
	Mark := Value and (1 shl Item) <> 0;
end;

procedure TCheckBoxes.Press (Item : Integer);
begin
	if Options and ofEditable <> 0 then Value := Value xor (1 shl Item);
end;

{******************************************************** TMultiCheckBoxes **}
{$IFNDEF Turbo6}
constructor TMultiCheckBoxes.Init (var Bounds : TRect; AStrings : PSItem; ASelRange : Byte;
									AFlags : Word; const AStates : String);
begin
	inherited Init (Bounds, AStrings);
{$ELSE}
constructor TMultiCheckBoxes.Init (var Bounds : TRect; AStrings : PSItem; ASelRange : Byte; AFlags : Word; AStates : String);
begin
	TCluster.Init (Bounds, AStrings);
{$ENDIF}
	SelRange := ASelRange;
	Flags := AFlags;
	States := NewStr (AStates);
end;

constructor TMultiCheckBoxes.Load (var S : TStream);
begin
{$IFNDEF Turbo6}
	inherited Load (S);
{$ELSE}
	TCluster.Load (S);
{$ENDIF}
	S.Read (SelRange, SizeOf (Byte));
	S.Read (Flags, SizeOf (Word));
	States := S.ReadStr;
end;

destructor TMultiCheckBoxes.Done;
begin
	DisposeStr (States);
{$IFNDEF Turbo6}
	inherited Done;
{$ELSE}
	TCluster.Done;
{$ENDIF}
end;

procedure TMultiCheckBoxes.Draw;
begin
	DrawMultiBox (DRIVERS._ChI, States^);
end;

function TMultiCheckBoxes.DataSize : Word;
begin
	DataSize := SizeOf (Longint);
end;

function TMultiCheckBoxes.MultiMark (Item : Integer) : Byte;
begin
	MultiMark := (Value shr (Word (Item) * WordRec (Flags).Hi))	and WordRec (Flags).Lo;
end;

procedure TMultiCheckBoxes.GetData (var Rec);
begin
	Longint (Rec) := Value;
end;

procedure TMultiCheckBoxes.Press (Item : Integer);
var
	CurState : ShortInt;
begin
	if Options and ofEditable = 0 then Exit;
	CurState := (Value shr (Word (Item) * WordRec (Flags).Hi)) and WordRec (Flags).Lo;
	Dec (CurState);
	if (CurState >= SelRange) or (CurState < 0) then
		CurState := SelRange - 1;
	Value := (Value and not (LongInt (WordRec (Flags).Lo)	shl (Word (Item) * WordRec (Flags).Hi)))
	or (LongInt (CurState) shl (Word (Item) * WordRec (Flags).Hi));
end;

procedure TMultiCheckBoxes.SetData (var Rec);
begin
	Value := Longint (Rec);
	DrawView;
end;

procedure TMultiCheckBoxes.Store (var S : TStream);
begin
{$IFNDEF Turbo6}
	inherited Store (S);
{$ELSE}
	TCluster.Store (S);
{$ENDIF}
	S.Write (SelRange, SizeOf (Byte));
	S.Write (Flags, SizeOf (Word));
	S.WriteStr (States);
end;

{**************************************************************** TListBox **}
type
	TListBoxRec = record
		List		: PCollection;
		Selection	: Word;
	end;

constructor TListBox.Init (var Bounds : TRect; ANumCols : Word; AScrollBar : PScrollBar);
var
	ARange	: Integer;
begin
{$IFNDEF Turbo6}
	inherited Init (Bounds, ANumCols, NIL, AScrollBar);
{$ELSE}
	TListViewer.Init (Bounds, ANumCols, NIL, AScrollBar);
{$ENDIF}
	List := NIL;
	SetRange (0);
end;

constructor TListBox.Load (var S : TStream);
begin
{$IFNDEF Turbo6}
	inherited Load (S);
{$ELSE}
	TListViewer.Load (S);
{$ENDIF}
	List := PCollection (S.Get);
end;

function TListBox.DataSize : Word;
begin
	DataSize := SizeOf (TListBoxRec);
end;

procedure TListBox.GetData (var Rec);
begin
	TListBoxRec (Rec).List := List;
	TListBoxRec (Rec).Selection := Focused;
end;

function TListBox.GetText (Item : Integer; MaxLen : Integer) : String;
begin
	if PtrRec (List).Seg <> 0 then
		GetText := PString (List^.At (Item))^
	else
		GetText := '';
end;

procedure TListBox.NewList (AList : PCollection);
begin
	PObject (List)^.Free;
	List := AList;
	if PtrRec (AList).Seg <> 0 then
		SetRange (AList^.Count)
	else
		SetRange (0);
	if Range > 0 then FocusItem (0);
	DrawView;
end;

procedure TListBox.SetData (var Rec);
begin
	NewList (TListBoxRec (Rec).List);
	FocusItem (TListBoxRec (Rec).Selection);
	DrawView;
end;

procedure TListBox.Store (var S : TStream);
begin
{$IFNDEF Turbo6}
	inherited Store (S);
{$ELSE}
	TListViewer.Store (S);
{$ENDIF}
	S.Put (List);
end;

{************************************************************* TStaticText **}
constructor TStaticText.Init (var Bounds : TRect; AText : String);
begin
{$IFNDEF Turbo6}
	inherited Init (Bounds);
{$ELSE}
	TView.Init (Bounds);
{$ENDIF}
	Text := NewStr (AText);
end;

constructor TStaticText.Load (var S : TStream);
begin
{$IFNDEF Turbo6}
	inherited Load (S);
{$ELSE}
	TView.Load (S);
{$ENDIF}
	Text := S.ReadStr;
end;

destructor TStaticText.Done;
begin
	DisposeStr (Text);
{$IFNDEF Turbo6}
	inherited Done;
{$ELSE}
	TView.Done;
{$ENDIF}
end;

procedure TStaticText.Draw;
var
	Color	: Word;
	Center	: Boolean;
	I, J, K,
	L, P, Y	: Integer;
	B		: TDrawBuffer;
	S		: String;
begin
	Color := GetColor ($0705);
	GetText (S);
	L := Length (S);
	P := 1;
	Y := 0;
	Center := False;
	while Y < Size.Y do
		begin
			MoveChar (B, ' ', Lo (Color), Size.X);
			if P <= L then
				begin
					if S [P] = #3 then
						begin
							Center := True;
							Inc (P);
						end;
					I := P;
					K := 0;
					repeat
						J := P;
						while (P <= L) and (S [P] = ' ') do Inc (P);
						while (P <= L) and (S [P] <> ' ') and (S [P] <> #13) do
							begin
								if S [P] = '~' then Inc (K);
								Inc (P);
							end;
					until (P > L) or (P >= I + Size.X - K) or (S [P] = #13);
					if P > I + Size.X then
						if J > I then P := J else P := I + Size.X;
					if Center then J := (Size.X - P + I + K) shr 1 else J := 0;
					MoveCBuf (B [J], S [I], Color, P - I - K);
					while (P <= L) and (S [P] = ' ') do Inc (P);
					if (P <= L) and (S [P] = #13) then
						begin
							Center := False;
							Inc (P);
							if (P <= L) and (S [P] = #10) then Inc (P);
						end;
				end;
		WriteLine (0, Y, Size.X, 1, B);
		Inc (Y);
	end;
end;

procedure TStaticText.GetText (var S : String);
begin
	if PtrRec (Text).Seg <> 0 then
		S := Text^
	else
		S := '';
end;

procedure TStaticText.Store (var S : TStream);
begin
{$IFNDEF Turbo6}
	inherited Store (S);
{$ELSE}
	TView.Store (S);
{$ENDIF}
	S.WriteStr (Text);
end;

{************************************************************** TParamText **}
constructor TParamText.Init (var Bounds : TRect; AText : String; AParamCount : Integer);
begin
{$IFNDEF Turbo6}
	inherited Init (Bounds, AText);
{$ELSE}
	TStaticText.Init (Bounds, AText);
{$ENDIF}
	ParamCount := AParamCount;
end;

constructor TParamText.Load (var S : TStream);
begin
{$IFNDEF Turbo6}
	inherited Load (S);
{$ELSE}
	TStaticText.Load (S);
{$ENDIF}
	S.Read (ParamCount, SizeOf (Integer));
end;

function TParamText.DataSize : Word;
begin
	DataSize := ParamCount * SizeOf (Longint);
end;

procedure TParamText.GetText (var S : String);
begin
	if PtrRec (Text).Seg <> 0 then
		FormatStr (S, Text^, ParamList^)
	else
		S := '';
end;

procedure TParamText.SetData (var Rec);
begin
	ParamList := @Rec;
end;

procedure TParamText.Store (var S : TStream);
begin
{$IFNDEF Turbo6}
	inherited Store (S);
{$ELSE}
	TStaticText.Store (S);
{$ENDIF}
	S.Write (ParamCount, SizeOf (Integer));
end;

{****************************************************************** TLabel **}
constructor TLabel.Init (var Bounds : TRect; AText : String; ALink : PView);
begin
{$IFNDEF Turbo6}
	inherited Init (Bounds, AText);
{$ELSE}
	TStaticText.Init (Bounds, AText);
{$ENDIF}
	Link := ALink;
	Options := Options or (ofPreProcess + ofPostProcess);
	EventMask := EventMask or evBroadcast;
end;

constructor TLabel.Load (var S : TStream);
begin
{$IFNDEF Turbo6}
	inherited Load (S);
{$ELSE}
	TStaticText.Load (S);
{$ENDIF}
	GetPeerViewPtr (S, Link);
end;

procedure TLabel.Draw;
var
	Color	: Word;
	B		: TDrawBuffer;
	SCOff	: TSpecialCharsUser;
begin
	if Light then
		begin
			Color := GetColor ($0707);
			SCOff := scuLabelFocus;
		end
	else
		begin
			Color := GetColor ($0705);
			SCOff := scuLabelNorm;
		end;
	MoveMarkCStr (B, Text^, Color, Size.X, SCOff);
	WriteLine (0, 0, Size.X, 1, B);
end;

procedure TLabel.HandleEvent (var Event : TEvent);
var
	C	: Char;

	procedure FocusLink;
	begin
		if (PtrRec (Link).Seg <> 0) and (Link^.Options and ofSelectable <> 0) then
			Link^.Focus;
		ClearEvent(Event);
	end;

begin
{$IFNDEF Turbo6}
	inherited HandleEvent (Event);
{$ELSE}
	TStaticText.HandleEvent (Event);
{$ENDIF}
	case Event.What of
		evMouseDown	: FocusLink;
		evKeyDown	:
			begin
				C := HotKey(Text^);
				if (GetAltCode(C) = Event.KeyCode) or ((C <> #0) and (Owner^.Phase = phPostProcess)
				and (KeyCase(Event.CharCode) = C)) then FocusLink
			end;
		evBroadcast	:
			if ((Event.Command = cmReceivedFocus) or (Event.Command = cmReleasedFocus))
			and	(PtrRec (Link).Seg <> 0) then
				begin
					Light := Link^.State and sfFocused <> 0;
					DrawView;
				end;
	end;
end;

procedure TLabel.Store (var S : TStream);
begin
{$IFNDEF Turbo6}
	inherited Store (S);
{$ELSE}
	TStaticText.Store (S);
{$ENDIF}
	PutPeerViewPtr (S, Link);
end;

{********************************************************** THistoryViewer **}
constructor THistoryViewer.Init (var Bounds : TRect; AHScrollBar, AVScrollBar : PScrollBar; AHistoryId : Word);
begin
{$IFNDEF Turbo6}
	inherited Init (Bounds, 1, AHScrollBar, AVScrollBar);
{$ELSE}
	TListViewer.Init (Bounds, 1, AHScrollBar, AVScrollBar);
{$ENDIF}
	HistoryId := AHistoryId;
	SetRange (HistoryCount (AHistoryId));
	with AHScrollBar^ do
		begin
			Flags := Flags or lfBoxes;
			SetRange (1, HistoryWidth);
		end;
end;

function THistoryViewer.GetText (Item : Integer; MaxLen : Integer) : String;
begin
	GetText := HistoryStr (HistoryId, Item);
end;

procedure THistoryViewer.HandleEvent (var Event : TEvent);
begin
	if ((Event.What = evMouseDown) and (Event.Double)) or ((Event.What = evKeyDown) and (Event.KeyCode = kbEnter)) then
		EndModal (cmOk)
	else
		if ((Event.What = evKeyDown) and (Event.KeyCode = kbEsc)) or ((Event.What = evCommand) and (Event.Command = cmCancel)) then
			EndModal (cmCancel)
		else
			begin
{$IFNDEF Turbo6}
				inherited HandleEvent (Event);
{$ELSE}
				TListViewer.HandleEvent (Event);
{$ENDIF}
				Exit;
			end;
	ClearEvent (Event);
end;

function THistoryViewer.HistoryWidth : Integer;
var
	Width,
	T,
	Count,
	I		: Integer;
begin
	Width := 0;
	Count := HistoryCount (HistoryId);
	for I := 0 to Count - 1 do
		begin
			T := Length (HistoryStr (HistoryId, I));
			if T > Width then Width := T;
		end;
	HistoryWidth := Width;
end;

{********************************************************** THistoryWindow **}
constructor THistoryWindow.Init (var Bounds : TRect; HistoryId : Word);
begin
{$IFNDEF Turbo6}
	inherited Init (Bounds, '', wnNoNumber);
{$ELSE}
	TWindow.Init (Bounds, '', wnNoNumber);
{$ENDIF}
	Flags := wfClose;
	InitViewer (HistoryId);
	Palette := 40;
end;

function THistoryWindow.GetSelection : String;
begin
	GetSelection := Viewer^.GetText (Viewer^.Focused, 255);
end;

procedure THistoryWindow.InitViewer (HistoryId : Word);
var
	R	: TRect;
begin
	GetExtent (R);
	R.Grow (-1, -1);
	Viewer := New (PHistoryViewer, Init (R, StandardScrollBar (sbHorizontal + sbHandleKeyboard),
					StandardScrollBar (sbVertical + sbHandleKeyboard), HistoryId));
	Insert (Viewer);
end;

{**************************************************************** THistory **}
constructor THistory.Init (var Bounds : TRect; ALink : PInputLine; AHistoryId : Word);
begin
{$IFNDEF Turbo6}
	inherited Init (Bounds);
{$ELSE}
	TView.Init (Bounds);
{$ENDIF}
	Options := Options or ofPostProcess;
	EventMask := EventMask or evBroadcast;
	Link := ALink;
	HistoryId := AHistoryId;
end;

constructor THistory.Load (var S : TStream);
begin
{$IFNDEF Turbo6}
	inherited Load (S);
{$ELSE}
	TView.Load (S);
{$ENDIF}
	GetPeerViewPtr (S, Link);
	S.Read (HistoryId, SizeOf (Word));
end;

procedure THistory.Draw;
var
	B	: TDrawBuffer;
begin
	MoveCStr (B, #222'~'#25'~'#221, GetColor ($1516));
	WriteLine (0, 0, Size.X, Size.Y, B);
end;

procedure THistory.HandleEvent (var Event : TEvent);
var
	HistoryWindow	: PHistoryWindow;
	R,
	P				: TRect;
	C				: Word;
	Rslt			: String;
begin
{$IFNDEF Turbo6}
	inherited HandleEvent (Event);
{$ELSE}
	TView.HandleEvent (Event);
{$ENDIF}
	if (Event.What = evMouseDown) or ((Event.What = evKeyDown) and (CtrlToArrow (Event.KeyCode) = kbDown)
	and (Link^.State and sfFocused <> 0)) then
		begin
			Link^.Select;
			HistoryAdd (HistoryId, Link^.Data^);
			Link^.GetBounds (R);
			Dec (R.A.X); Inc (R.B.X); Inc (R.B.Y, 7); Dec (R.A.Y, 1);
			Owner^.GetExtent (P);
			R.Intersect (P);
			Dec (R.B.Y, 1);
			HistoryWindow := InitHistoryWindow (R);
			if PtrRec (HistoryWindow).Seg <> 0 then
				begin
					C := Owner^.ExecView (HistoryWindow);
					if C = cmOk then
						begin
							Rslt := HistoryWindow^.GetSelection;
							if Length (Rslt) > Link^.MaxLen then Rslt [0] := Char (Link^.MaxLen);
							Link^.Data^ := Rslt;
							Link^.SelectAll (True);
							Link^.DrawView;
						end;
				end;
			PObject (HistoryWindow)^.Free;
			ClearEvent (Event);
		end
	else
		if (Event.What = evBroadcast) then
			if ((Event.Command = cmReleasedFocus) and (Event.InfoPtr = Link)) or (Event.Command = cmRecordHistory) then
				HistoryAdd (HistoryId, Link^.Data^);
end;

function THistory.InitHistoryWindow (var Bounds : TRect) : PHistoryWindow;
var
	P	: PHistoryWindow;
begin
	P := New (PHistoryWindow, Init (Bounds, HistoryId));
	P^.HelpCtx := Link^.HelpCtx;
	InitHistoryWindow := P;
end;

procedure THistory.Store (var S : TStream);
begin
{$IFNDEF Turbo6}
	inherited Store (S);
{$ELSE}
	TView.Store (S);
{$ENDIF}
	PutPeerViewPtr (S, Link);
	S.Write (HistoryId, SizeOf (Word));
end;

{****************************************** Dialogs registration procedure **}
procedure RegisterDialogs;
begin
	RegisterType (RDialog);
	RegisterType (RInputLine);
	RegisterType (RButton);
	RegisterType (RCluster);
	RegisterType (RRadioButtons);
	RegisterType (RCheckBoxes);
	RegisterType (RMultiCheckBoxes);
	RegisterType (RListBox);
	RegisterType (RStaticText);
	RegisterType (RLabel);
	RegisterType (RHistory);
	RegisterType (RParamText);
end;

end.
