{1/2 Copyright by Leonid Youriev 1992, 93}
{$I VERSION}

unit StdDlg;

interface

uses	Objects, Drivers, Views, Dialogs, Dos, FileServ, CharLog;

{$I VERSION}

const

{ Commands }
	cmFileOpen			= 800; { Returned from TFileDialog when Open pressed }
	cmFileReplace		= 801; { Returned from TFileDialog when Replace pressed }
	cmFileClear			= 802; { Returned from TFileDialog when Clear pressed }
	cmFileInit			= 803; { Used by TFileDialog internally }
	cmChangeDir			= 804; { Used by TChDirDialog internally }
	cmRevert			= 805; { Used by TChDirDialog internally }

{ Messages }
	cmFileFocused		= 806; { A new file was focused in the TFileList }
	cmFileDoubleClicked	= 807; { A file was selected in the TFileList }

{ Flags }
	fdOkButton			= $0001; { Put an OK button in the dialog }
	fdOpenButton		= $0002; { Put an Open button in the dialog }
	fdReplaceButton		= $0004; { Put a Replace button in the dialog }
	fdClearButton		= $0008; { Put a Clear button in the dialog }
	fdHelpButton		= $0010; { Put a Help button in the dialog }
	fdNoLoadDir			= $0100; { Do not load the current directory }

	cdNormal			= $0000; { Option to use dialog immediately }
	cdNoLoadDir			= $0001; { Option to init the dialog to store on a stream }
	cdHelpButton		= $0002; { Put a help button in the dialog }

type

	PFileInputLine = ^TFileInputLine;
	TFileInputLine = Object (TInputLine)
		constructor	Init (Var Bounds : TRect; AMaxLen : Integer);
		procedure	HandleEvent (Var Event : TEvent); virtual;
	end;

	PFileCollection = ^TFileCollection;
	TFileCollection = Object (TSortedCollection)
		function	Compare (Key1, Key2 : Pointer) : Integer; virtual;
		procedure	FreeItem (Item : Pointer); virtual;
		function	GetItem (Var S : TStream) : Pointer; virtual;
		procedure	PutItem (Var S : TStream; Item : Pointer); virtual;
	end;

	PSortedListBox = ^TSortedListBox;
	TSortedListBox = Object (TListBox)
		SearchPos	: Word;
		Shift		: Byte;
		constructor	Init (Var Bounds : TRect; ANumCols : Word; AScrollBar : PScrollBar);
		procedure	HandleEvent (Var Event : TEvent); virtual;
		function	GetKey (Var S : String) : Pointer; virtual;
		procedure	NewList (AList : PCollection); virtual;
	end;

	PFileList = ^TFileList;
	TFileList = Object (TSortedListBox)
		constructor	Init (Var Bounds : TRect; AWildCard : PathStr; AScrollBar : PScrollBar);
		destructor	Done; virtual;
		function	DataSize : Word; virtual;
		procedure	FocusItem (Item : Integer); virtual;
		procedure	GetData (Var Rec); virtual;
		function	GetText (Item : Integer; MaxLen : Integer) : String; virtual;
		function	GetKey (Var S : String) : Pointer; virtual;
		procedure	HandleEvent (Var Event : TEvent); virtual;
		procedure	ReadDirectory (AWildCard : PathStr);
		procedure	SetData (Var Rec); virtual;
	end;

	PFileInfoPane = ^TFileInfoPane;
	TFileInfoPane = Object (TView)
		S			: TSearchRec;
		constructor	Init (Var Bounds : TRect);
		procedure	Draw; virtual;
		procedure	HandleEvent (Var Event : TEvent); virtual;
	end;

	TWildStr	= PathStr;

	PFileDialog = ^TFileDialog;
	TFileDialog = Object (TDialog)
		FileName	: PFileInputLine;
		FileList	: PFileList;
		WildCard	: TWildStr;
		Directory	: PString;
		constructor	Init (AWildCard : TWildStr; ATitle : String; InputName : String; AOptions : Word; HistoryId : Byte);
		constructor	Load (Var S : TStream);
		destructor	Done; virtual;
		procedure	GetData (Var Rec); virtual;
		procedure	GetFileName (Var S : PathStr);
		procedure	HandleEvent (Var Event : TEvent); virtual;
		procedure	SetData (Var Rec); virtual;
		procedure	Store (Var S : TStream);
		function	Valid (Command : Word) : Boolean; virtual;
	private
		procedure ReadDirectory;
	end;

	PDirEntry = ^TDirEntry;
	TDirEntry = record
		DisplayText	: PString;
		Directory	: PString;
	end;

	PDirCollection = ^TDirCollection;
	TDirCollection = Object (TCollection)
		function	GetItem (Var S : TStream) : Pointer; virtual;
		procedure	FreeItem (Item : Pointer); virtual;
		procedure	PutItem (Var S : TStream; Item : Pointer); virtual;
	end;

	PDirListBox = ^TDirListBox;
	TDirListBox = Object (TListBox)
		Dir			: DirStr;
		Cur			: Word;
		constructor	Init (Var Bounds : TRect; AScrollBar : PScrollBar);
		destructor	Done; virtual;
		function	GetText (Item : Integer; MaxLen : Integer) : String; virtual;
		procedure	HandleEvent (Var Event : TEvent); virtual;
		function	IsSelected (Item : Integer) : Boolean; virtual;
		procedure	NewDirectory (Var ADir : DirStr);
		procedure	SetState (AState : Word; Enable : Boolean); virtual;
	end;

	PChDirDialog = ^TChDirDialog;
	TChDirDialog = Object (TDialog)
		DirInput	: PInputLine;
		DirList		: PDirListBox;
		OkButton	: PButton;
		ChDirButton	: PButton;
		constructor	Init (AOptions : Word; HistoryId : Word);
		constructor	Load (Var S : TStream);
		function	DataSize : Word; virtual;
		procedure	GetData (Var Rec); virtual;
		procedure	HandleEvent (Var Event : TEvent); virtual;
		procedure	SetData (Var Rec); virtual;
		procedure	Store (Var S : TStream);
		function	Valid (Command : Word) : Boolean; virtual;
	private
		procedure SetUpDialog;
	end;

const
	CInfoPane = #30;

	RFileInputLine	: TStreamRec = (
		ObjType	: 60;
		VmtLink	: Ofs (TypeOf (TFileInputLine)^);
		Load	: @TFileInputLine.Load;
		Store	: @TFileInputLine.Store);

	RFileCollection	: TStreamRec = (
		ObjType	: 61;
		VmtLink	: Ofs (TypeOf (TFileCollection)^);
		Load	: @TFileCollection.Load;
		Store	: @TFileCollection.Store);

	RFileList		: TStreamRec = (
		ObjType	: 62;
		VmtLink	: Ofs (TypeOf (TFileList)^);
		Load	: @TFileList.Load;
		Store	: @TFileList.Store);

	RFileInfoPane	: TStreamRec = (
		ObjType	: 63;
		VmtLink	: Ofs (TypeOf (TFileInfoPane)^);
		Load	: @TFileInfoPane.Load;
		Store	: @TFileInfoPane.Store);

	RFileDialog		: TStreamRec = (
		ObjType	: 64;
		VmtLink	: Ofs (TypeOf (TFileDialog)^);
		Load	: @TFileDialog.Load;
		Store	: @TFileDialog.Store);

	RDirCollection	: TStreamRec = (
		ObjType	: 65;
		VmtLink	: Ofs (TypeOf (TDirCollection)^);
		Load	: @TDirCollection.Load;
		Store	: @TDirCollection.Store);

	RDirListBox		: TStreamRec = (
		ObjType	: 66;
		VmtLink	: Ofs (TypeOf (TDirListBox)^);
		Load	: @TDirListBox.Load;
		Store	: @TDirListBox.Store);

	RChDirDialog	: TStreamRec = (
		ObjType	: 67;
		VmtLink	: Ofs (TypeOf (TChDirDialog)^);
		Load	: @TChDirDialog.Load;
		Store	: @TChDirDialog.Store);

	RSortedListBox	: TStreamRec = (
		ObjType	: 68;
		VmtLink	: Ofs (TypeOf (TSortedListBox)^);
		Load	: @TSortedListBox.Load;
		Store	: @TSortedListBox.Store);

procedure RegisterStdDlg;

implementation

uses	App, Memory, HistList, MsgBox, SysVars;

type
	PSearchRec = ^TSearchRec;

{********************************************************** TFileInputLine **}
constructor TFileInputLine.Init (Var Bounds : TRect; AMaxLen : Integer);
begin
	TInputLine.Init (Bounds, AMaxLen);
	EventMask := EventMask or evBroadcast;
end;

procedure TFileInputLine.HandleEvent (Var Event : TEvent);
Var
	Dir		: DirStr;
	Name	: NameStr;
	Ext		: ExtStr;
begin
	TInputLine.HandleEvent (Event);
	if (Event.What = evBroadcast) and (Event.Command = cmFileFocused) and (State and sfSelected = 0) then
		begin
			if PSearchRec (Event.InfoPtr)^.Attr and Directory <> 0 then
				Data^ := PSearchRec (Event.InfoPtr)^.Name + '\' + PFileDialog (Owner)^.WildCard
			else
				Data^ := PSearchRec (Event.InfoPtr)^.Name;
			DrawView;
		end;
end;

{********************************************************* TFileCollection **}
function TFileCollection.Compare (Key1, Key2 : Pointer) : Integer;
begin
	if PSearchRec (Key1)^.Name = PSearchRec (Key2)^.Name then
		Compare := 0
	else
		if PSearchRec (Key1)^.Name = '..' then
			Compare := 1
		else
			if PSearchRec (Key2)^.Name = '..' then
				Compare := -1
			else
				if (PSearchRec (Key1)^.Attr and Directory <> 0) and (PSearchRec (Key2)^.Attr and Directory = 0) then
					Compare := 1
				else
					if (PSearchRec (Key2)^.Attr and Directory <> 0) and (PSearchRec (Key1)^.Attr and Directory = 0) then
						Compare := -1
					else
						if PSearchRec (Key1)^.Name > PSearchRec (Key2)^.Name then
							Compare := 1
						else
							Compare := -1;
end;

procedure TFileCollection.FreeItem (Item : Pointer);
begin
	Dispose (PSearchRec (Item));
end;

function TFileCollection.GetItem (Var S : TStream) : Pointer;
Var
	Item	: PSearchRec;
begin
	New (Item);
	S.Read (Item^, SizeOf (TSearchRec));
	GetItem := Item;
end;

procedure TFileCollection.PutItem (Var S : TStream; Item : Pointer);
begin
	S.Write (Item^, SizeOf (TSearchRec));
end;

{********************************************************** TSortedListBox **}
constructor TSortedListBox.Init (Var Bounds : TRect; ANumCols : Word; AScrollBar : PScrollBar);
begin
	TListBox.Init (Bounds, ANumCols, AScrollBar);
	SearchPos := 0;
	ShowCursor;
	SetCursor (1, 0);
end;

procedure TSortedListBox.HandleEvent (Var Event : TEvent);
Var
	CurString,
	NewString	: String;
	K			: Pointer;
	Value,
	OldPos,
	OldValue	: Integer;
	T			: Boolean;
begin
	OldValue := Focused;
	TListBox.HandleEvent (Event);
	if OldValue <> Focused then
		SearchPos := 0;
	if Event.What = evKeyDown then
		begin
			if Event.CharCode <> #0 then
				begin
					Value := Focused;
					if Value < Range then
						CurString := GetText (Value, 255)
					else
						CurString := '';
					OldPos := SearchPos;
					if Event.KeyCode = kbBack then
						begin
							if SearchPos = 0 then Exit;
								Dec (SearchPos);
							if SearchPos = 0 then Shift := ShiftState;
							CurString [0] := Char (SearchPos);
						end
					else
						if (Event.CharCode = '.') then
							SearchPos := Pos ('.', CurString)
						else
							begin
								Inc (SearchPos);
								if SearchPos = 1 then Shift := ShiftState;
								CurString [0] := Char (SearchPos);
								CurString [SearchPos] := Event.CharCode;
							end;
					K := GetKey (CurString);
					T := PSortedCollection (List)^.Search (K, Value);
					if Value < Range then
						begin
							if Value < Range then
								NewString := GetText (Value, 255)
							else
								NewString := '';
							if Equal (NewString, CurString, SearchPos) then
								begin
									if Value <> OldValue then
										begin
											FocusItem (Value);
											SetCursor (Cursor.X + SearchPos, Cursor.Y);
										end
									else
										SetCursor (Cursor.X + (SearchPos - OldPos), Cursor.Y);
								end
							else
								SearchPos := OldPos;
						end
					else
						SearchPos := OldPos;
					if (SearchPos <> OldPos) or (Event.CharCode in ['A'..'Z', 'a'..'z']) then
						ClearEvent (Event);
				end;
		end;
end;

function TSortedListBox.GetKey (Var S : String) : Pointer;
begin
	GetKey := @S;
end;

procedure TSortedListBox.NewList (AList : PCollection);
begin
	TListBox.NewList (AList);
	SearchPos := 0;
end;

{*************************************************************** TFileList **}
constructor TFileList.Init (Var Bounds : TRect; AWildCard : PathStr; AScrollBar : PScrollBar);
begin
	TSortedListBox.Init (Bounds, 2, AScrollBar);
end;

destructor TFileList.Done;
begin
	PObject (List)^.Free;
	TListBox.Done;
end;

function TFileList.DataSize : Word;
begin
	DataSize := 0;
end;

procedure TFileList.FocusItem (Item : Integer);
begin
	TSortedListBox.FocusItem (Item);
	if Focused < Range then
		Message (Owner, evBroadcast, cmFileFocused, List^.At (Focused));
end;

procedure TFileList.GetData (Var Rec);
begin
end;

function TFileList.GetKey (Var S : String) : Pointer;
const
	SR	: TSearchRec = ();

begin
	if (Shift and $03 <> 0) or ((S <> '') and (S [1]='.')) then
		SR.Attr := Directory
	else
		SR.Attr := 0;
	SR.Name := S;
	UpStr (SR.Name);
	GetKey := @SR;
end;

function TFileList.GetText (Item : Integer; MaxLen : Integer) : String;
Var
	S	: String;
	SR	: PSearchRec;
begin
	SR := PSearchRec (List^.At (Item));
	S := SR^.Name;
	if SR^.Attr and Directory <> 0 then
		begin
			S [Length (S) + 1] := '\';
			Inc (S [0]);
		end;
	GetText := S;
end;

procedure TFileList.HandleEvent (Var Event : TEvent);
begin
	if (Event.What = evMouseDown) and (Event.Double) then
		begin
			SendCommand (cmOk);
			ClearEvent (Event);
		end
	else
		TSortedListBox.HandleEvent (Event);
end;

procedure TFileList.ReadDirectory (AWildCard : PathStr);
const
	FindAttr	= ReadOnly + Archive;
	AllFiles	= '*.*';
	PrevDir		= '..';
Var
	S			: SearchRec;
	P			: PSearchRec;
	FileList	: PFileCollection;
	NumFiles	: Word;
	CurPath		: PathStr;
	Dir			: DirStr;
	Name		: NameStr;
	Ext			: ExtStr;
	Event		: TEvent;
	Tmp			: PathStr;
	Flag		: Integer;
begin
	NumFiles := 0;
	AWildCard := FExpand (AWildCard);
	FSplit (AWildCard, Dir, Name, Ext);
	FileList := New (PFileCollection, Init (5, 5));
	FindFirst (AWildCard, FindAttr, S);
	P := @P;
	while (PtrRec (P).Seg <> 0) and (DosError = 0) do
		begin
			if (S.Attr and Directory = 0) then
				begin
					P := MemAlloc (SizeOf (P^));
					if PtrRec (P).Seg <> 0 then
						begin
							Move (S.Attr, P^, SizeOf (P^));
							FileList^.Insert (P);
						end;
				end;
			FindNext (S);
		end;
	Tmp := Dir + AllFiles;
	FindFirst (Tmp, Directory, S);
	while (PtrRec (P).Seg <> 0) and (DosError = 0) do
		begin
			if (S.Attr and Directory <> 0) and (S.Name [1] <> '.') then
				begin
					P := MemAlloc (SizeOf (P^));
					if PtrRec (P).Seg <> 0 then
						begin
							Move (S.Attr, P^, SizeOf (P^));
							FileList^.Insert (PObject (P));
						end;
				end;
			FindNext (S);
		end;
	if Length (Dir) > 4 then
		begin
			P := MemAlloc (SizeOf (P^));
			if PtrRec (P).Seg <> 0 then
				begin
					FindFirst (Tmp, Directory, S);
					FindNext (S);
					if (DosError = 0) and (S.Name = PrevDir) then
						Move (S.Attr, P^, SizeOf (P^))
					else
						With P^ do
							begin
								Name := PrevDir;
								Size := 0;
								Time := $210000;
								Attr := Directory;
							end;
					FileList^.Insert (PObject (P));
				end;
		end;
	if PtrRec (P).Seg = 0 then MessageBox (#3'祭  䠩.', NIL, mfCancelButton + mfWarning);
	NewList (FileList);
	if List^.Count > 0 then
		begin
			Event.What := evBroadcast;
			Event.Command := cmFileFocused;
			Event.InfoPtr := List^.At (0);
			Owner^.HandleEvent (Event);
		end;
end;

procedure TFileList.SetData (Var Rec);
begin
	with PFileDialog (Owner)^ do
		Self.ReadDirectory (Directory^ + WildCard);
end;

{*********************************************************** TFileInfoPane **}
constructor TFileInfoPane.Init (Var Bounds : TRect);
begin
	TView.Init (Bounds);
	EventMask := EventMask or evBroadcast;
end;

procedure TFileInfoPane.Draw;
Var
	B		: TDrawBuffer;
	D		: String [10];
	M		: String [4];
	Color	: Byte;
	Time	: DateTime;
	Path	: PathStr;
	FmtId	: String;
	Params	: array [0..6] of LongInt;
	Str		: String[80];
const
	sDirectoryLine	= ' %-12s %-12s %2d %-4s %4d %2d:%02d ';
	sFileLine		= ' %-12s %-12d %2d %-4s %4d %2d:%02d ';
	Month	: array [1..12] of String [4] =	('', '', '', '', '', '', '', '', '', '', '', '');
begin
	Path := FExpand (PFileDialog (Owner)^.Directory^ + PFileDialog (Owner)^.WildCard);
	Color := Lo (GetColor ($1D));
	MoveChar (B, ' ', Color, Size.X);
	MoveStr (B [1], Path, Color);
	WriteLine (0, 0, Size.X, 1, B);
	Params [0] := LongInt (@S.Name);
	if S.Attr and Directory <> 0 then
		begin
			FmtId := sDirectoryLine;
			D := '⠫';
			Params [1] := LongInt (@D);
		end
	else
		begin
			FmtId := sFileLine;
			Params [1] := S.Size;
		end;
	UnpackTime (S.Time, Time);
	M := Month [Time.Month];
	Params [3] := LongInt (@M);
	Params [2] := Time.Day;
	Params [4] := Time.Year;
	Params [5] := Time.Hour mod 24;
	Params [6] := Time.Min;
	FormatStr (Str, FmtId, Params);
	MoveStr (B, Str, Color);
	WriteLine (0, 1, Size.X, 1, B);
	MoveChar (B, ' ', Color, Size.X);
	WriteLine (0, 2, Size.X, Size.Y - 2, B);
end;

procedure TFileInfoPane.HandleEvent (Var Event : TEvent);
begin
	TView.HandleEvent (Event);
	if (Event.What = evBroadcast) and (Event.Command = cmFileFocused) then
		begin
			S := PSearchRec (Event.InfoPtr)^;
			DrawView;
		end;
end;

{************************************************************* TFileDialog **}
constructor TFileDialog.Init (AWildCard : TWildStr; ATitle : String; InputName : String; AOptions : Word; HistoryId : Byte);
Var
	Control	: PView;
	R		: TRect;
	S		: String;
	Opt		: Word;
	ACurDir	: PathStr;
begin
	R.Assign (15, 1, 64, 20);
	TDialog.Init (R, ATitle);
	Options := Options or ofCentered;
	WildCard := AWildCard;
	R.Assign (3, 3, 31, 4);
	FileName := New (PFileInputLine, Init (R, 79));
	FileName^.Data^ := WildCard;
	Labeled (InputName + '^', FileName);
	R.Assign (31, 3, 34, 4);
	Control := New (PHistory, Init (R, FileName, HistoryId));
	Insert (Control);
	R.Assign (3, 14, 34, 15);
	Control := New (PScrollBar, Init (R, lfDefault));
	Insert (Control);
	R.Assign (3, 6, 34, 14);
	FileList := New (PFileList, Init (R, WildCard, PScrollBar (Control)));
	Labeled ('&', FileList);
	R.Assign (35, 3, 46, 5);
	Opt := bfDefault;
	if AOptions and fdOpenButton <> 0 then
		begin
			Insert (New (PButton, Init (R, '&', cmFileOpen, Opt)));
			Opt := bfNormal;
			Inc (R.A.Y, 3); Inc (R.B.Y, 3);
		end;
	if AOptions and fdOkButton <> 0 then
		begin
			Insert (New (PButton, Init (R, cOkStr, cmFileOpen, Opt)));
			Opt := bfNormal;
			Inc (R.A.Y, 3); Inc (R.B.Y, 3);
		end;
	if AOptions and fdReplaceButton <> 0 then
		begin
			Insert (New (PButton, Init (R, '&', cmFileReplace, Opt)));
			Opt := bfNormal;
			Inc (R.A.Y, 3); Inc (R.B.Y, 3);
		end;
	if AOptions and fdClearButton <> 0 then
		begin
			Insert (New (PButton, Init (R, '&', cmFileClear, Opt)));
			Opt := bfNormal;
			Inc (R.A.Y, 3); Inc (R.B.Y, 3);
		end;
	Insert (New (PButton, Init (R, cCancelStr, cmCancel, bfNormal)));
	Inc (R.A.Y, 3); Inc (R.B.Y, 3);
	if AOptions and fdHelpButton <> 0 then
		begin
			Insert (New (PButton, Init (R, cHelpStr, cmHelp, bfNormal)));
			Inc (R.A.Y, 3); Inc (R.B.Y, 3);
		end;
	R.Assign (1, 16, 48, 18);
	Control := New (PFileInfoPane, Init (R));
	Insert (Control);
	SelectNext (False);
	if AOptions and fdNoLoadDir = 0 then ReadDirectory;
end;

constructor TFileDialog.Load (Var S : TStream);
Var
	ACurDir	: DirStr;
	ViewId	: Word;
begin
	TDialog.Load (S);
	S.Read (WildCard, SizeOf (TWildStr));
	GetSubViewPtr (S, FileName);
	GetSubViewPtr (S, FileList);
	ReadDirectory;
end;

destructor TFileDialog.Done;
begin
	DisposeStr (Directory);
	TDialog.Done;
end;

procedure TFileDialog.GetData (Var Rec);
begin
	GetFilename (PathStr (Rec));
end;

procedure TFileDialog.GetFileName (Var S : PathStr);
Var
	Path	: PathStr;
	Name	: NameStr;
	Ext		: ExtStr;
	TPath	: PathStr;
	TName	: NameStr;
	TExt	: NameStr;

	function RelativePath (Var S : PathStr) : Boolean;
	Var
		I,
		J	: Integer;
		P	: PathStr;
	begin
		RemoveLowSpace (S);
		if (S <> '') and ((S [1] = '\') or (S [2] = ':')) then
			RelativePath := False
		else
			RelativePath := True;
	end;

	function NoWildChars (S : String) : String; near; assembler;
	asm
		PUSH	DS
		LDS		SI, S
		XOR		AX, AX
		LODSB
		XCHG	AX, CX
		LES		DI, @Result
		INC		DI
		JCXZ	@3
	@1:	LODSB
		CMP		AL, '?'
		JE		@2
		CMP		AL, '*'
		JE		@2
		STOSB
	@2:	LOOP	@1
	@3:	XCHG	AX, DI
		MOV		DI, WORD PTR @Result
		SUB		AX, DI
		DEC		AX
		STOSB
		POP		DS
end;

begin
	S := FileName^.Data^;
	if RelativePath (S) then
		S := FExpand (Directory^ + S)
	else
		S := FExpand (S);
	FSplit (S, Path, Name, Ext);
	if ((Name = '') or (Ext = '')) and not IsDir (S) then
		begin
			FSplit (WildCard, TPath, TName, TExt);
			if ((Name = '') and (Ext = '')) then
				S := Path + TName + TExt
			else
				if Name = '' then
					S := Path + TName + Ext
				else
					if Ext = '' then
						begin
							if IsWild (Name) then
								S := Path + Name + TExt
							else
								S := Path + Name + NoWildChars (TExt);
						end;
		end;
end;

procedure TFileDialog.HandleEvent (Var Event : TEvent);
begin
	TDialog.HandleEvent (Event);
	if Event.What = evCommand then
		case Event.Command of
			cmFileOpen,
			cmFileReplace,
			cmFileClear	:
				begin
					EndModal (Event.Command);
					ClearEvent (Event);
				end;
		end;
end;

procedure TFileDialog.SetData (Var Rec);
begin
	TDialog.SetData (Rec);
	if (PathStr (Rec) <> '') and (IsWild (TWildStr (Rec))) then
		begin
			Valid (cmFileInit);
			FileName^.Select;
		end;
end;

procedure TFileDialog.ReadDirectory;
begin
	FileList^.ReadDirectory (WildCard);
	Directory := NewStr (GetCurDir);
end;

procedure TFileDialog.Store (Var S : TStream);
begin
	TDialog.Store (S);
	S.Write (WildCard, SizeOf (TWildStr));
	PutSubViewPtr (S, FileName);
	PutSubViewPtr (S, FileList);
end;

function TFileDialog.Valid (Command : Word) : Boolean;
Var
	T		: Boolean;
	FName	: PathStr;
	Dir		: DirStr;
	Name	: NameStr;
	Ext		: ExtStr;

	function CheckDirectory (Var S : PathStr) : Boolean;
	begin
		if not PathValid (S) then
			begin
				MessageBox (#3'   ⠫.', NIL, mfError + mfCancelButton);
				FileName^.Select;
				CheckDirectory := False;
			end
		else
			CheckDirectory := True;
	end;

begin
	if Command = cmValid then
		begin
			Valid := True;
			Exit;
		end
	else
		Valid := False;
	if TDialog.Valid (Command) then
		begin
			GetFileName (FName);
			if (Command <> cmCancel) and (Command <> cmFileClear) then
				begin
					if IsWild (FName) then
						begin
							FSplit (FName, Dir, Name, Ext);
							if CheckDirectory (Dir) then
								begin
									DisposeStr (Directory);
									Directory := NewStr (Dir);
									WildCard := Name + Ext;
									if Command <> cmFileInit then FileList^.Select;
									FileList^.ReadDirectory (Directory^ + WildCard);
								end
						end
					else
						if IsDir (FName) then
							begin
								if CheckDirectory (FName) then
									begin
										DisposeStr (Directory);
										Directory := NewSTr (FName + '\');
										if Command <> cmFileInit then FileList^.Select;
										FileList^.ReadDirectory (Directory^ + WildCard);
									end
							end
						else
							if ValidFileName (FName) then
								Valid := True
							else
								begin
									MessageBox (#3'⨬  䠩.', NIL, mfError + mfCancelButton);
									Valid := False;
								end
				end
			else
				Valid := True;
		end;
end;

{********************************************************** TDirCollection **}
function TDirCollection.GetItem (Var S : TStream) : Pointer;
Var
	DirItem	: PDirEntry;
begin
	New (DirItem);
	DirItem^.DisplayText := S.ReadStr;
	DirItem^.Directory := S.ReadStr;
	GetItem := DirItem;
end;

procedure TDirCollection.FreeItem (Item : Pointer);
Var
	DirItem	: PDirEntry absolute Item;
begin
	DisposeStr (DirItem^.DisplayText);
	DisposeStr (DirItem^.Directory);
	Dispose (DirItem);
end;

procedure TDirCollection.PutItem (Var S : TStream; Item : Pointer);
Var
	DirItem	: PDirEntry absolute Item;
begin
	S.WriteStr (DirItem^.DisplayText);
	S.WriteStr (DirItem^.Directory);
end;

{************************************************************* TDirListBox **}
const
	DrivesS	: String [5] = '᪨';
	Drives	: PString = @DrivesS;

constructor TDirListBox.Init (Var Bounds : TRect; AScrollBar : PScrollBar);
begin
	TListBox.Init (Bounds, 1, AScrollBar);
	Dir := '';
end;

destructor TDirListBox.Done;
begin
	PObject (List)^.Free;
	TListBox.Done;
end;

function TDirListBox.GetText (Item : Integer; MaxLen : Integer) : String;
begin
	GetText := PDirEntry (List^.At (Item))^.DisplayText^;
end;

procedure TDirListBox.HandleEvent (Var Event : TEvent);
begin
	if (Event.What = evMouseDown) and (Event.Double) then
		begin
			SendCommand (cmChangeDir);
			ClearEvent (Event);
		end
	else
		TListBox.HandleEvent (Event);
end;

function TDirListBox.IsSelected (Item : Integer) : Boolean;
begin
	IsSelected := Item = Cur;
end;

procedure TDirListBox.NewDirectory (Var ADir : DirStr);
const
	PathDir		= '';
	FirstDir	= '';
	MiddleDir	= ' ';
	LastDir		= ' ';
	IndentSize	= ' ';
Var
	AList		: PCollection;
	NewDir,
	Dirct		: DirStr;
	C,
	OldC		: Char;
	S,
	Indent		: String [80];
	P			: PString;
	isFirst		: Boolean;
	SR			: SearchRec;
	I			: Integer;
	DirEntry	: PDirEntry;

	function NewDirEntry (DisplayText, Directory : String) : PDirEntry; near;
	Var
		DirEntry	: PDirEntry;
	begin
		New (DirEntry);
		DirEntry^.DisplayText := NewStr (DisplayText);
		DirEntry^.Directory := NewStr (Directory);
		NewDirEntry := DirEntry;
	end;

begin
	Dir := ADir;
	AList := New (PDirCollection, Init (5, 5));
	AList^.Insert (NewDirEntry (Drives^, Drives^));
	if Dir = Drives^ then
		begin
			isFirst := True;
			OldC := ' ';
			for C := 'A' to 'Z' do
				begin
					if (C < 'C') or DriveValid (C) then
						begin
							if OldC <> ' ' then
								begin
									if isFirst then
										begin
											S := FirstDir + OldC;
											isFirst := False;
										end
									else
										S := MiddleDir + OldC;
									AList^.Insert (NewDirEntry (S, OldC + ':\'));
								end;
							if C = GetCurDrive then Cur := AList^.Count;
							OldC := C;
						end;
				end;
			if OldC <> ' ' then AList^.Insert (NewDirEntry (LastDir + OldC, OldC + ':\'));
		end
	else
		begin
			Indent := IndentSize;
			NewDir := Dir;
			Dirct := Copy (NewDir, 1, 3);
			AList^.Insert (NewDirEntry (PathDir + Dirct, Dirct));
			NewDir := Copy (NewDir, 4, 255);
			while NewDir <> '' do
				begin
					I := Pos ('\', NewDir);
					if I <> 0 then
						begin
							S := Copy (NewDir, 1, I - 1);
							Dirct := Dirct + S;
							AList^.Insert (NewDirEntry (Indent + PathDir + S, Dirct));
							NewDir := Copy (NewDir, I + 1, 255);
						end
					else
						begin
							Dirct := Dirct + NewDir;
							AList^.Insert (NewDirEntry (Indent + PathDir + NewDir, Dirct));
							NewDir := '';
						end;
					Indent := Indent + IndentSize;
					Dirct := Dirct + '\';
				end;
			Cur := AList^.Count - 1;
			isFirst := True;
			NewDir := Dirct + '*.*';
			FindFirst (NewDir, Directory, SR);
			while DosError = 0 do
				begin
					if (SR.Attr and Directory <> 0) and (SR.Name [1] <> '.') then
						begin
							if isFirst then
								begin
									S := FirstDir;
									isFirst := False;
								end
							else
								S := MiddleDir;
							AList^.Insert (NewDirEntry (Indent + S + SR.Name, Dirct + SR.Name));
						end;
					FindNext (SR);
				end;
			P := PDirEntry (AList^.At (AList^.Count - 1))^.DisplayText;
			I := Pos ('', P^);
			if I = 0 then
				begin
					I := Pos ('', P^);
					if I <> 0 then
						P^ [I] := '';
				end
			else
				begin
					P^ [I + 1] := '';
					P^ [I + 2] := '';
				end;
		end;
	NewList (AList);
	FocusItem (Cur);
end;

procedure TDirListBox.SetState (AState : Word; Enable : Boolean);
begin
	TListBox.SetState (AState, Enable);
	if AState and sfFocused <> 0 then
		PChDirDialog (Owner)^.ChDirButton^.MakeDefault (Enable);
end;

{************************************************************ TChDirDialog **}
constructor TChDirDialog.Init (AOptions : Word; HistoryId : Word);
Var
	R		: TRect;
	Control	: PView;
	CurDir	: DirStr;
begin
	R.Assign (16, 2, 64, 20);
	TDialog.Init (R, ' ⠫');
	Options := Options or ofCentered;
	R.Assign (3, 3, 30, 4);
	DirInput := New (PInputLine, Init (R, 68));
	Labeled ('& ⠫', DirInput);
	R.Assign (30, 3, 33, 4);
	Control := New (PHistory, Init (R, DirInput, HistoryId));
	Insert (Control);
	R.Assign (32, 6, 33, 16);
	Control := New (PScrollBar, Init (R, lfDefault));
	Insert (Control);
	R.Assign (3, 6, 32, 16);
	DirList := New (PDirListBox, Init (R, PScrollBar (Control)));
	Labeled ('&ॢ', DirList);
	R.Assign (35, 6, 45, 8);
	OkButton := New (PButton, Init (R, cOkStr, cmOK, bfDefault));
	Insert (OkButton);
	Inc (R.A.Y, 3); Inc (R.B.Y, 3);
	ChDirButton := New (PButton, Init (R, '&', cmChangeDir, bfNormal));
	Insert (ChDirButton);
	Inc (R.A.Y, 3); Inc (R.B.Y, 3);
	Insert (New (PButton, Init (R, '&', cmRevert, bfNormal)));
	if AOptions and cdHelpButton <> 0 then
		begin
			Inc (R.A.Y, 3); Inc (R.B.Y, 3);
			Insert (New (PButton, Init (R, cHelpStr, cmHelp, bfNormal)));
		end;
	if AOptions and cdNoLoadDir = 0 then SetUpDialog;
	SelectNext (False);
end;

constructor TChDirDialog.Load (Var S : TStream);
Var
	CurDir	: DirStr;
begin
	TDialog.Load (S);
	GetSubViewPtr (S, DirList);
	GetSubViewPtr (S, DirInput);
	GetSubViewPtr (S, OkButton);
	GetSubViewPtr (S, ChDirbutton);
	SetUpDialog;
end;

function TChDirDialog.DataSize : Word;
begin
	DataSize := 0;
end;

procedure TChDirDialog.GetData (Var Rec);
begin
end;

procedure TChDirDialog.HandleEvent (Var Event : TEvent);
Var
	CurDir	: DirStr;
	P		: PDirEntry;
begin
	TDialog.HandleEvent (Event);
	case Event.What of
		evCommand	:
			begin
				case Event.Command of
					cmRevert	: GetDir (0, CurDir);
					cmChangeDir	:
						begin
							P := DirList^.List^.At (DirList^.Focused);
							if (P^.Directory^ = Drives^) or DriveValid (P^.Directory^ [1]) then
								CurDir := P^.Directory^
							else
								Exit;
						end;
				else
					Exit;
				end;
				if (Length (CurDir) > 3) and (CurDir [Length (CurDir)] = '\') then
					CurDir := Copy (CurDir, 1, Length (CurDir) - 1);
				DirList^.NewDirectory (CurDir);
				DirInput^.Data^ := CurDir;
				DirInput^.DrawView;
				DirList^.Select;
				ClearEvent (Event);
			end;
	end;
end;

procedure TChDirDialog.SetData (Var Rec);
begin
end;

procedure TChDirDialog.SetUpDialog;
Var
	CurDir	: DirStr;
begin
	if PtrRec (DirList).Seg <> 0 then
		begin
			CurDir := GetCurDir;
			DirList^.NewDirectory (CurDir);
			if (Length (CurDir) > 3) and (CurDir [Length (CurDir)] = '\') then
				CurDir := Copy (CurDir, 1, Length (CurDir) - 1);
			if PtrRec (DirInput).Seg <> 0 then
				begin
					DirInput^.Data^ := CurDir;
					DirInput^.DrawView;
				end;
		end;
end;

procedure TChDirDialog.Store (Var S : TStream);
begin
	TDialog.Store (S);
	PutSubViewPtr (S, DirList);
	PutSubViewPtr (S, DirInput);
	PutSubViewPtr (S, OkButton);
	PutSubViewPtr (S, ChDirButton);
end;

function TChDirDialog.Valid (Command : Word) : Boolean;
Var
	P	: PathStr;
begin
	Valid := True;
	if Command = cmOk then
		begin
			P := FExpand (DirInput^.Data^);
			if (Length (P) > 3) and (P [Length (P)] = '\') then Dec (P [0]);
			ChDir (P);
			if IOResult <> 0 then
				begin
					MessageBox (#3' ⠫.', NIL, mfError + mfCancelButton);
					Valid := False;
				end;
		end;
end;

procedure RegisterStdDlg;
begin
	RegisterType (RFileInputLine);
	RegisterType (RFileCollection);
	RegisterType (RFileList);
	RegisterType (RFileInfoPane);
	RegisterType (RFileDialog);
	RegisterType (RDirCollection);
	RegisterType (RDirListBox);
	RegisterType (RSortedListBox);
	RegisterType (RChDirDialog);
end;

end.
